@@ -51,13 +51,12 @@ function UIAnnualReport:UIAnnualReport(ui, world)
   self.rep_amount = 0
 
   if not pcall(function()
-    local palette   = gfx:loadPalette("QData", "Award02V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
+    local palette = gfx:loadPalette("QData", "Award02V.pal", true)
 
     -- Right now the statistics are first
     --self.background = gfx:loadRaw("Fame01V", 640, 480)
     self.award_background = gfx:loadRaw("Award01V", 640, 480)
-    self.stat_background = gfx:loadRaw("Award02V", 640, 480)
+    self.stat_background = gfx:loadRaw("Award02V", 640, 480, "QData", "QData", "Award02V.pal", true)
     self.background = self.stat_background
 
     self.stat_font = gfx:loadFont("QData", "Font45V", false, palette)
@@ -347,7 +346,7 @@ function UIAnnualReport:addTrophy(text, award_type, amount)
     trophy_parts.info = {
       text = text,
       award_type = award_type,
-      amount = amount
+      amount = math.floor(amount)
     }
 
     local --[[persistable:annual_report_show_trophy_motivation]] function change() self:showTrophyMotivation(no) end
@@ -383,7 +382,7 @@ function UIAnnualReport:addAward(text, award_type, amount)
     award_parts.info = {
       text = text,
       award_type = award_type,
-      amount = amount
+      amount = math.floor(amount)
     }
 
     -- The plaque
@@ -502,7 +501,12 @@ function UIAnnualReport:draw(canvas, x, y)
   local font = self.stat_font
   local world = self.ui.app.world
 
-  if self.state == 1 then -- Fame screen
+  if self.state == 1 then -- Fame/Shame screen (High Scores)
+    -- TODO: This screen should be displayed at the start of the annual report, but currently
+    -- it is not shown, likely as the code for this being unfinished. When implemented
+    -- we only show this screen at the very start, and once they go to the next screen
+    -- it no longer becomes accessible.
+
     -- Title and column names
     font:draw(canvas, _S.high_score.best_scores, x + 220, y + 104, 200, 0)
     font:draw(canvas, _S.high_score.pos, x + 218, y + 132)
@@ -514,13 +518,14 @@ function UIAnnualReport:draw(canvas, x, y)
     local dy = 0
     --for i = 1, 10 do
       font:draw(canvas, i .. ".", x + 220, y + 160 + dy)
-      font:draw(canvas, world.hospitals[1].name:upper(), x + 260, y + 160 + dy)
+      font:draw(canvas, world:getLocalPlayerHospital().name:upper(), x + 260, y + 160 + dy)
       font:draw(canvas, "NA", x + 360, y + 160 + dy)
       -- dy = dy + 25
     --end
   elseif self.state == 2 then -- Statistics screen
     self:drawStatisticsScreen(canvas, x, y)
   else -- Award and trophy screen
+    -- Note: Each 'amount' is wrapped by string.format("%.0f", x) to persist integer notation
     -- Write out motivation if appropriate
     if self.trophy_motivation then
       -- If it is a plaque showing we write in stone text.
@@ -533,7 +538,7 @@ function UIAnnualReport:draw(canvas, x, y)
       end
       self.stone_font:draw(canvas, award_type, x + 220, y + 330, 200, 0)
       -- Amount won/lost
-      self.stone_font:draw(canvas, "+" .. info.amount, x + 220, y + 355, 200, 0)
+      self.stone_font:draw(canvas, string.format("+%.0f", info.amount), x + 220, y + 355, 200, 0)
     elseif self.award_motivation then
       local info = self.awards[self.award_motivation].info
       self.write_font:drawWrapped(canvas, info.text, x + 235, y + 125, 165, "center")
@@ -544,11 +549,7 @@ function UIAnnualReport:draw(canvas, x, y)
       end
       self.write_font:draw(canvas, award_type, x + 220, y + 290, 200, 0)
       -- The amount won/lost
-      local text = ""
-      if info.amount > 0 then
-        text = "+"
-      end
-      self.write_font:draw(canvas, text .. info.amount, x + 220, y + 315, 200, 0)
+      self.write_font:draw(canvas, string.format("%+.0f", info.amount), x + 220, y + 315, 200, 0)
     end
   end
 end
@@ -588,6 +589,7 @@ function UIAnnualReport:drawStatisticsScreen(canvas, x, y)
   for i, hospital in ipairs(world.hospitals) do
     local name = hospital.name
 
+    -- Note: Each 'value' is wrapped by string.format("%.0f", x) to persist integer notation
     -- Most Money
     local index_m = getindex(self.money_sort, i)
     -- index_* is the returned value of the sorted place for this player.
@@ -595,42 +597,59 @@ function UIAnnualReport:drawStatisticsScreen(canvas, x, y)
     -- duplicate has been found, one additional row lower is the right place to be.
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_dy * (index_m - 1))
-    font:draw(canvas, self.money_sort[index_m].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.money_sort[index_m].value), x + 240,
         y + row_y + row_dy * (index_m - 1), 70, 0, "right")
 
     -- Highest Salary
     local index_s = getindex(self.salary_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_dy * (index_s - 1))
-    font:draw(canvas, self.salary_sort[index_s].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.salary_sort[index_s].value), x + 240 + col_x,
         y + row_y + row_dy * (index_s - 1), 70, 0, "right")
 
     -- Most Cures
     local index_c = getindex(self.cures_sort, i)
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_no_y + row_dy * (index_c - 1))
-    font:draw(canvas, self.cures_sort[index_c].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.cures_sort[index_c].value), x + 240,
         y + row_y + row_no_y + row_dy * (index_c - 1), 70, 0, "right")
 
     -- Most Deaths
     local index_d = getindex(self.deaths_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_no_y + row_dy * (index_d - 1))
-    font:draw(canvas, self.deaths_sort[index_d].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.deaths_sort[index_d].value), x + 240 + col_x,
         y + row_y + row_no_y + row_dy * (index_d - 1), 70, 0, "right")
 
     -- Most Visitors
     local index_v = getindex(self.visitors_sort, i)
     font:draw(canvas, name:upper(), x + 140,
         y + row_y + row_no_y * 2 + row_dy * (index_v - 1))
-    font:draw(canvas, self.visitors_sort[index_v].value, x + 240,
+    font:draw(canvas, string.format("%.0f", self.visitors_sort[index_v].value), x + 240,
         y + row_y + row_no_y * 2 + row_dy * (index_v - 1), 70, 0, "right")
 
     -- Highest Value
     local index_v2 = getindex(self.value_sort, i)
     font:draw(canvas, name:upper(), x + 140 + col_x,
         y + row_y + row_no_y * 2 + row_dy * (index_v2 - 1))
-    font:draw(canvas, self.value_sort[index_v2].value, x + 240 + col_x,
+    font:draw(canvas, string.format("%.0f", self.value_sort[index_v2].value), x + 240 + col_x,
         y + row_y + row_no_y * 2 + row_dy * (index_v2 - 1), 70, 0, "right")
   end
 end
+
+function UIAnnualReport:afterLoad(old, new)
+  if old < 179 then
+    local gfx = TheApp.gfx
+
+    local palette = gfx:loadPalette("QData", "Award02V.pal", true)
+    self.award_background = gfx:loadRaw("Award01V", 640, 480)
+    self.stat_background = gfx:loadRaw("Award02V", 640, 480, "QData", "QData", "Award02V.pal", true)
+    self.background = self.stat_background
+    self.stat_font = gfx:loadFont("QData", "Font45V", false, palette)
+    self.write_font = gfx:loadFont("QData", "Font47V", false, palette)
+    self.stone_font = gfx:loadFont("QData", "Font46V", false, palette)
+    self.panel_sprites = gfx:loadSpriteTable("QData", "Award03V", true, palette)
+  end
+
+  UIFullscreen.afterLoad(self, old, new)
+end
