@@ -34,10 +34,9 @@ function UITownMap:UITownMap(ui)
   self.app = app
 
   if not pcall(function()
-    local palette   = gfx:loadPalette("QData", "Town01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
+    local palette = gfx:loadPalette("QData", "Town01V.pal", true)
 
-    self.background = gfx:loadRaw("Town01V", 640, 480)
+    self.background = gfx:loadRaw("Town01V", 640, 480, "QData", "QData", "Town01V.pal", true)
     self.info_font  = gfx:loadFont("QData", "Font34V", false, palette)
     self.city_font = gfx:loadFont("QData", "Font31V", false, palette)
     self.money_font = gfx:loadFont("QData", "Font05V")
@@ -108,7 +107,7 @@ function UITownMap:close()
 end
 
 local flag_cache = {}
-function UITownMap:onMouseMove(x, y)
+function UITownMap:onMouseMove(x, y, dx, dy)
   local tx = math.floor((x - 227) / 3)
   local ty = math.floor((y - 25) / 3)
   self.hover_plot = nil
@@ -345,3 +344,16 @@ function UITownMap:bankStats()
   self.ui:addWindow(dlg)
   self.ui:getWindow(UIBottomPanel):updateButtonStates()
 end
+
+function UITownMap:afterLoad(old, new)
+  if old < 179 then
+    local gfx = TheApp.gfx
+    local palette = gfx:loadPalette("QData", "Town01V.pal", true)
+    self.background = gfx:loadRaw("Town01V", 640, 480, "QData", "QData", "Town01V.pal", true)
+    self.info_font = gfx:loadFont("QData", "Font34V", false, palette)
+    self.city_font = gfx:loadFont("QData", "Font31V", false, palette)
+    self.panel_sprites = gfx:loadSpriteTable("QData", "Town02V", true, palette)
+  end
+
+  UIFullscreen.afterLoad(self, old, new)
+end
