@@ -78,7 +78,7 @@ end
 
 
 function UIOptions:UIOptions(ui, mode)
-  self:UIResizable(ui, 320, 430, col_bg)
+  self:UIResizable(ui, 320, 460, col_bg)
 
   local app = ui.app
   self.mode = mode
@@ -120,6 +120,17 @@ function UIOptions:UIOptions(ui, mode)
   self:addBevelPanel(80, title_y_pos, 165, 20, col_caption):setLabel(_S.options_window.caption)
     .lowered = true
 
+  -- Check for updates
+  local updates_y_pos = self:_getOptionYPos()
+  local updates_string = app.config.check_for_updates and
+      _S.options_window.option_enabled or _S.options_window.option_disabled
+  self:addBevelPanel(20, updates_y_pos, BTN_WIDTH, BTN_HEIGHT, col_shadow, col_bg, col_bg)
+      :setLabel(_S.options_window.check_for_updates):setTooltip(_S.tooltip.options_window.check_for_updates).lowered = true
+  self.updates_panel =
+      self:addBevelPanel(165, updates_y_pos, BTN_WIDTH, BTN_HEIGHT, col_bg):setLabel(updates_string)
+  self.updates_button = self.updates_panel:makeToggleButton(0, 0, 140, BTN_HEIGHT, nil, self.buttonUpdates)
+      :setToggleState(app.config.check_for_updates)
+
   -- Fullscreen
   local fullscreen_y_pos = self:_getOptionYPos()
   self:addBevelPanel(20, fullscreen_y_pos, BTN_WIDTH, BTN_HEIGHT, col_shadow, col_bg, col_bg)
@@ -292,7 +303,21 @@ function UIOptions:selectResolution(number)
   end
 end
 
-function UIOptions:buttonFullscreen(checked)
+--! Changes check for update setting to on/of
+function UIOptions:toggleUpdateCheck()
+  self.ui.app.config.check_for_updates = not self.ui.app.config.check_for_updates
+  self.ui.app:saveConfig()
+end
+
+--! Function handles button toggle of checking for updates
+function UIOptions:buttonUpdates()
+  self:toggleUpdateCheck()
+  local new_updates_string = self.ui.app.config.check_for_updates and
+      _S.options_window.option_enabled or _S.options_window.option_disabled
+  self.updates_panel:setLabel(new_updates_string)
+end
+
+function UIOptions:buttonFullscreen()
   if not self.ui:toggleFullscreen() then
       local err = {_S.errors.unavailable_screen_size}
       self.ui:addWindow(UIInformation(self.ui, err))
@@ -301,7 +326,7 @@ function UIOptions:buttonFullscreen(checked)
   self.fullscreen_panel:setLabel(self.ui.app.fullscreen and _S.options_window.option_on or _S.options_window.option_off)
 end
 
-function UIOptions:buttonMouseCapture(checked)
+function UIOptions:buttonMouseCapture()
   local app = self.ui.app
   app.config.capture_mouse = not app.config.capture_mouse
   app:saveConfig()
@@ -336,7 +361,7 @@ function UIOptions:buttonBrowseForTHInstall()
   self.ui:addWindow(browser)
 end
 
-function UIOptions:buttonAudioGlobal(checked)
+function UIOptions:buttonAudioGlobal()
   local app = self.ui.app
   app.config.audio = not app.config.audio
   app:saveConfig()
@@ -653,4 +678,4 @@ function UIZoomSpeed:close(ok)
   else
     self.callback(self.ui.app.config.zoom_speed)
   end
-end
\ No newline at end of file
+end
