@@ -83,6 +83,7 @@ end
 function Patient:setDisease(disease)
   self.disease = disease
   disease.initPatient(self)
+  self.th:setPatientEffect(disease.effect or AnimationEffect.None)
   self.diagnosed = false
   self.diagnosis_progress = 0
   self.cure_rooms_visited = 0
@@ -92,8 +93,6 @@ function Patient:setDisease(disease)
   for i, room in ipairs(self.disease.diagnosis_rooms) do
     self.available_diagnosis_rooms[i] = room
   end
-  -- Decide an insurance company, one out of four patients have one.
-  -- TODO: May need some balancing, but it is roughly the same as in TH.
   local company = math.random(1,12)
   if company < 4 then
     self.insurance_company = company
@@ -269,6 +268,9 @@ function Patient:treatDisease()
 
   hospital:receiveMoneyForTreatment(self)
 
+  -- Remove visual effects of disease.
+  self.th:setPatientEffect(AnimationEffect.None)
+
   -- Either the patient is no longer sick, or he/she dies.
   if self:isTreatmentEffective() then
     self:cure()
@@ -336,13 +338,22 @@ function Patient:die()
     self:setNextAction(MeanderAction():setCount(1))
   end
   self:queueAction(DieAction())
-  self:updateDynamicInfo(_S.dynamic_info.patient.actions.dying)
+  self:setDynamicInfoText(_S.dynamic_info.patient.actions.dying)
 end
 
+-- Actions we can interrupt for in the canPeeOrPuke function
+local good_actions = {walk=true, idle=true, seek_room=true, queue=true}
+
 function Patient:canPeeOrPuke(current)
-  return ((current.name == "walk" or current.name == "idle" or
-           current.name == "seek_room" or current.name == "queue") and
-      not self.going_home and self.world.map.th:getCellFlags(self.tile_x, self.tile_y).buildable)
+  if not good_actions[current.name] then return false end
+  if self.going_home then return false end
+
+  local th = self.world.map.th
+  local cell_flags = th:getCellFlags(self.tile_x, self.tile_y)
+  if not cell_flags.buildable or not cell_flags.hospital then return false end
+
+  local parcel = cell_flags.parcelId
+  return parcel ~= 0 and th:getPlotOwner(parcel) == self.hospital:getPlayerIndex()
 end
 
 --! Animations for when there is an earth quake
@@ -381,9 +392,7 @@ function Patient:fallingAnnounce()
   (_A.warnings.falling_5),
   (_A.warnings.falling_6),
   }
-  if msg then
-    self.world.ui.adviser:say(msg[math.random(1, #msg)])
-  end
+  self.hospital:giveAdvice(msg)
 end
 
 --! Perform 'shake fist' action.
@@ -416,7 +425,7 @@ function Patient:pee()
     self:changeAttribute("happiness", -0.02) -- not being able to find a loo and doing it in the corridor will make you sad too
     if not self.hospital.did_it_on_floor then
       self.hospital.did_it_on_floor = true
-      self.world.ui.adviser:say(_A.warnings.people_did_it_on_the_floor)
+      self.hospital:giveAdvice({_A.warnings.people_did_it_on_the_floor})
     end
   else
     return
@@ -468,7 +477,7 @@ function Patient:goHome(reason, disease_id)
     self:setMood("cured", "activate")
     self:changeAttribute("happiness", 0.8)
     hosp:updateCuredCounts(self)
-    self:updateDynamicInfo(_S.dynamic_info.patient.actions.cured)
+    self:setDynamicInfoText(_S.dynamic_info.patient.actions.cured)
 
   elseif reason == "kicked" then
     self:setMood("exit", "activate")
@@ -479,7 +488,7 @@ function Patient:goHome(reason, disease_id)
     self:changeAttribute("happiness", -0.5)
 
     local treatment_name = self.hospital.disease_casebook[disease_id].disease.name
-    self.world.ui.adviser:say(_A.warnings.patient_not_paying:format(treatment_name))
+    self.hospital:giveAdvice({_A.warnings.patient_not_paying:format(treatment_name)})
     hosp:updateNotCuredCounts(self, reason)
     self:clearDynamicInfo()
     self:setDynamicInfo('text', {"", _S.dynamic_info.patient.actions.prices_too_high})
@@ -527,6 +536,14 @@ function Patient:despawn()
   Humanoid.despawn(self)
 end
 
+-- Checks if the room the patient is in is the final treatment room for their disease
+-- NB: Check diagnosed status as some illnesses use Psych for diagnosis and treatment
+function Patient:_checkIfCureRoom(room)
+  local num_rooms = #self.disease.treatment_rooms
+  return (room and self.disease.treatment_rooms[num_rooms] == room.room_info.id
+      and self.diagnosed)
+end
+
 -- This function handles changing of the different attributes of the patient.
 -- For example if thirst gets over a certain level (now: 0.7), the patient
 -- tries to find a drinks machine nearby.
@@ -538,10 +555,10 @@ function Patient:tickDay()
       self:goHome("kicked")
       if self.diagnosed then
         -- No treatment rooms
-        self:updateDynamicInfo(_S.dynamic_info.patient.actions.no_treatment_available)
+        self:setDynamicInfoText(_S.dynamic_info.patient.actions.no_treatment_available)
       else
         -- No diagnosis rooms
-        self:updateDynamicInfo(_S.dynamic_info.patient.actions.no_diagnoses_available)
+        self:setDynamicInfoText(_S.dynamic_info.patient.actions.no_diagnoses_available)
       end
     elseif self.waiting == 10 then
       self:tapFoot()
@@ -567,20 +584,22 @@ function Patient:tickDay()
   if not Humanoid.tickDay(self) then
     return
   end
+
   -- Die before we poo or drink
-  -- patient has been in the hospital for over 6 months and is still not well, so will become sad and will either get fed up and leave
-  -- or stay in the hope that you will cure them before they die
-  -- strange, but in TH happiness does not go down, even when close to death IMO that is wrong as you would be unhappy if you waited too long.
-  -- TODO death animation for slack female is missing its head. For now the only option is for her to get fed up and leave
-  -- this can be changed when the animation thing is resolved
+  -- Patient has been in the hospital for over 6 months and is still not well, so
+  -- will become sad and will either get fed up and leave or stay in the hope that you
+  -- will cure them before they die.
+  -- Note: This behaviour intentionally differs from Theme Hospital where being close to
+  -- death does not impact happiness.
   -- TODO clean up this block, nonmagical numbers
   if self.attributes["health"] >= 0.18 and self.attributes["health"] < 0.22 then
     self:setMood("sad2", "activate")
     self:changeAttribute("happiness", -0.0002) -- waiting too long will make you sad
-    -- There is a 1/3 chance that the patient will get fed up and leave
-    -- note, this is potentially run 10 ((0.22-0.18)/0.004) times, hence the 1/30 chance.
-    if math.random(1,30) == 1 then
-      self:updateDynamicInfo(_S.dynamic_info.patient.actions.fed_up)
+    -- There is a 1/3 chance that the patient will get fed up and leave.
+    -- This is potentially run 10 ((0.22-0.18)/0.004) times, hence the 1/30 chance.
+    -- If patient is already in the cure room, let the treatment happen.
+    if not self:_checkIfCureRoom(self:getRoom()) and math.random(1,30) == 1 then
+      self:setDynamicInfoText(_S.dynamic_info.patient.actions.fed_up)
       self:setMood("sad2", "deactivate")
       self:goHome("kicked")
     end
@@ -599,20 +618,17 @@ function Patient:tickDay()
   elseif self.attributes["health"] >= 0.01 and self.attributes["health"] < 0.06 then
     self:setMood("sad5", "deactivate")
     self:setMood("sad6", "activate")
-  -- its not looking good
+  -- it's not looking good
   elseif self.attributes["health"] > 0.00 and self.attributes["health"] < 0.01 then
     self.attributes["health"] = 0.0
   -- is there time to say a prayer
   elseif self.attributes["health"] == 0.0 then
-    -- people who are in a room should not die:
-    -- 1. they are being cured in this moment. dying in the last few seconds
-    --    before the cure makes only a subtle difference for gameplay
-    -- 2. they will leave the room soon (toilets, diagnostics) and will die then
     if not self:getRoom() and not self:getCurrentAction().is_leaving then
       self:setMood("sad6", "deactivate")
       self:die()
     end
-    --dead people aren't thirsty
+    -- Patient died, will die when they leave the room, will be cured, or is leaving
+    -- the hospital. Regardless we do not need to adjust any other attribute
     return
   end
 
@@ -694,7 +710,7 @@ function Patient:tickDay()
     end
   end)
   -- It always makes you happy to see you are in safe place
-  self.world:findObjectNear(self, "extinguisher", 2, function(x, y)
+  self.world:findObjectNear(self, "extinguisher", 2, function()
     self:changeAttribute("happiness", 0.0002)
   end)
   -- sitting makes you happy whilst standing and walking does not
@@ -877,7 +893,7 @@ function Patient:setTile(x, y)
 
           -- A callout is only needed if there are no handymen employed
           if self.hospital:countStaffOfCategory("Handyman", 1) == 0 then
-            self.world.ui.adviser:say(_A.staff_advice.need_handyman_litter)
+            self.hospital:giveAdvice({_A.staff_advice.need_handyman_litter})
           end
         end
       end
@@ -925,17 +941,17 @@ function Patient:addToTreatmentHistory(room)
   end
 end
 
-function Patient:updateDynamicInfo(action_string)
-  -- Retain the old text if only an update is wanted, i.e. no new string is supplied.
-  if action_string == nil then
-    if self.action_string then
-      action_string = self.action_string
-    else
-      action_string = ""
-    end
-  else
-    self.action_string = action_string
-  end
+--! Sets the dynamic text currently in use by the patient
+--!param action_string (string) What text to say in dynamic text
+function Patient:setDynamicInfoText(action_string)
+  self.action_string = action_string
+  self:updateDynamicInfo()
+end
+
+--! Updates a patient's dynamic info
+--! Can be called direct to refresh only
+function Patient:updateDynamicInfo()
+  local action_string = self.action_string or ""
   local info = ""
   if self.going_home then
     self:setDynamicInfo('progress', nil)
@@ -965,16 +981,14 @@ function Patient:updateDynamicInfo(action_string)
   end
   -- Set the centre line of dynamic info based on contagiousness, if appropriate
   local epidemic = self.hospital and self.hospital.epidemic
-  if epidemic and epidemic.coverup_in_progress then
-    if self.infected and not self.vaccinated then
-      self:setDynamicInfo('text',
-        {action_string, _S.dynamic_info.patient.actions.epidemic_contagious, info})
-    elseif self.vaccinated then
+  if epidemic and self.infected and epidemic.coverup_in_progress then
+    if self.vaccinated then
       self:setDynamicInfo('text',
         {action_string, _S.dynamic_info.patient.actions.epidemic_vaccinated, info})
     else
-      self:setDynamicInfo('text', {action_string, "", info})
-    end
+      self:setDynamicInfo('text',
+        {action_string, _S.dynamic_info.patient.actions.epidemic_contagious, info})
+	end
   else
     self:setDynamicInfo('text', {action_string, "", info})
   end
@@ -1011,21 +1025,14 @@ function Patient:updateMessage(choice)
             output_text = strings.need_to_build:format(room_name)
           end
         elseif #req.rooms == 0 and next(req.staff) then
-          local staffclass_to_string = {
-              Nurse        = _S.staff_title.nurse,
-              Doctor       = _S.staff_title.doctor,
-              Surgeon      = _S.staff_title.surgeon,
-              Psychiatrist = _S.staff_title.psychiatrist,
-          }
-          output_text = strings.need_to_employ:format(staffclass_to_string[next(req.staff)])
+          output_text = strings.need_to_employ:format(StaffProfile.translateStaffClass(next(req.staff)))
         end
         self.message[3].text = output_text
       else
         -- no requirements missing
         enabled = false
       end
-    else -- if choice == "guess_cure" then
-      -- TODO: implement
+    -- TODO: implement choice == "guess_cure"
     end
 
     for _, c in ipairs(self.message.choices) do
@@ -1095,14 +1102,41 @@ function Patient:afterLoad(old, new)
       self.going_to_toilet = "no"
     end
   end
+  if old < 158 then
+    -- Always overwrite patient effect from old saves since it is
+    -- loaded from a dummy field which used to have another meaning.
+    if self.disease and not self.cured then
+      self.th:setPatientEffect(self.disease.effect or AnimationEffect.None)
+    else
+      self.th:setPatientEffect(AnimationEffect.None)
+    end
+  end
+  -- fix walks into room - regression in #2086
+  if old < 177 then
+    for i, action in ipairs(self.action_queue) do
+      if action.name == "walk" and action.is_entering then
+        action.saved_must_happen = false
+        -- if its not the currently executing action
+        -- we should set must_happen to false
+        if i > 1 then
+          action.must_happen = false
+        end
+      end
+    end
+  end
+  self:updateDynamicInfo()
   Humanoid.afterLoad(self, old, new)
 end
 
 function Patient:isMalePatient()
-  if string.find(self.humanoid_class,"Female") then return false
-  elseif string.find(self.humanoid_class,"Male") then return true
+  if string.find(self.humanoid_class, "Female") then return false
+  elseif string.find(self.humanoid_class, "Male") then return true
   else
-    local male_patient_classes = {["Chewbacca Patient"] = true,["Elvis Patient"] = true,["Invisible Patient"] = true}
+    local male_patient_classes = {
+      ["Chewbacca Patient"] = true,
+      ["Elvis Patient"] = true,
+      ["Invisible Patient"] = true
+    }
     return male_patient_classes[self.humanoid_class] ~= nil
   end
 end
@@ -1113,39 +1147,38 @@ end
 !param meander_before_resume (bool) Indicator to insert an additional action before resuming.
 ]]
 function Patient:interruptAndRequeueAction(current_action, queue_pos, meander_before_resume)
-  if current_action.name == "idle" or current_action.name == "walk" then
-    local requeue_action = {
-        name = current_action.name,
-        x = current_action.x,
-        y = current_action.y,
-        must_happen = current_action.must_happen,
-        is_entering = current_action.is_entering,
-    }
-    self:queueAction(requeue_action, queue_pos)
-    -- If we were idling, also go away a little before continuing with
-    -- that important action.
-    if current_action.name == "idle" and meander_before_resume then
+  -- Rebuild the current action.
+  local requeue_action
+  if current_action.name == "idle" then
+    requeue_action = IdleAction():setMustHappen(current_action.must_happen)
+    requeue_action:setDirection(current_action.direction)
+
+    -- Go away a little before continuing with that important action.
+    if meander_before_resume then
       self:queueAction(MeanderAction():setCount(1), queue_pos)
-    -- if walking, and action was already started, use the original must_happen
-    elseif current_action.name == "walk" then
-      if current_action.saved_must_happen ~= nil then
-        requeue_action.must_happen = current_action.saved_must_happen
-      end
-      -- need to copy the reserve_on_resume, otherwise the new queued action will not
-      -- unreserve on interrupt
-      requeue_action.reserve_on_resume = current_action.reserve_on_resume
     end
+  elseif current_action.name == "walk" then
+    requeue_action = WalkAction(current_action.x, current_action.y)
+    requeue_action:setMustHappen(current_action.must_happen)
+    if current_action.saved_must_happen ~= nil then
+      requeue_action.must_happen = current_action.saved_must_happen
+    end
+    -- need to copy the reserve_on_resume, otherwise the new queued action will not
+    -- unreserve on interrupt
+    requeue_action.reserve_on_resume = current_action.reserve_on_resume
+    requeue_action:setIsEntering(current_action.is_entering)
   else
     -- We were seeking a room, start that action from the beginning
     -- i.e. do not set the must_happen flag.
-    self:queueAction({
-      name = current_action.name,
-      room_type = current_action.room_type,
-      message_sent = true,
-      diagnosis_room = current_action.diagnosis_room,
-      treatment_room = current_action.treatment_room,
-    }, queue_pos)
+    requeue_action = SeekRoomAction(current_action.room_type)
+    if current_action.treatment_room then requeue_action:enableTreatmentRoom() end
+    if current_action.diagnosis_room then
+      requeue_action:setDiagnosisRoom(current_action.diagnosis_room)
+    end
+    requeue_action.message_sent = true
   end
+  self:queueAction(requeue_action, queue_pos)
+
   -- now interrupt
   if current_action.on_interrupt then
     current_action.on_interrupt(current_action, self)
@@ -1155,4 +1188,4 @@ function Patient:interruptAndRequeueAction(current_action, queue_pos, meander_be
 end
 
 -- Dummy callback for savegame compatibility
-local callbackNewRoom = --[[persistable:patient_toilet_build_callback]] function(room) end
+local callbackNewRoom = --[[persistable:patient_toilet_build_callback]] function() end
