@@ -20,6 +20,8 @@ SOFTWARE. --]]
 
 corsixth.require("announcer")
 
+local AnnouncementPriority = _G["AnnouncementPriority"]
+
 --! A Doctor, Nurse, Receptionist, Handyman, or Surgeon
 class "Staff" (Humanoid)
 
@@ -31,6 +33,8 @@ function Staff:Staff(...)
   self:Humanoid(...)
   self.hover_cursor = TheApp.gfx:loadMainCursor("staff")
   self.parcelNr = 0
+  self.leave_sounds = {}
+  self.leave_priority = AnnouncementPriority.High
 end
 
 --! Handle daily adjustments to staff.
@@ -70,20 +74,20 @@ function Staff:tickDay()
     end
   end)
   -- It always makes you happy to see you are in safe place
-  self.world:findObjectNear(self, "extinguisher", 2, function(x, y)
+  self.world:findObjectNear(self, "extinguisher", 2, function()
     self:changeAttribute("happiness", 0.002)
   end)
   -- Extra room items add to your happiness (some more than others)
-  self.world:findObjectNear(self, "bin", 2, function(x, y)
+  self.world:findObjectNear(self, "bin", 2, function()
     self:changeAttribute("happiness", 0.001)
   end)
-  self.world:findObjectNear(self, "bookcase", 2, function(x, y)
+  self.world:findObjectNear(self, "bookcase", 2, function()
     self:changeAttribute("happiness", 0.003)
   end)
-  self.world:findObjectNear(self, "skeleton", 2, function(x, y)
+  self.world:findObjectNear(self, "skeleton", 2, function()
     self:changeAttribute("happiness", 0.002)
   end)
-  self.world:findObjectNear(self, "tv", 2, function(x, y)
+  self.world:findObjectNear(self, "tv", 2, function()
     self:changeAttribute("happiness", 0.0005)
   end)
   -- Being able to rest from work and play the video game or pool will make you happy
@@ -184,8 +188,7 @@ function Staff:checkIfWaitedTooLong()
       if self.hospital.policies.grant_wage_increase then
         local amount = math.floor(math.max(self.profile.wage * 1.1, (self.profile:getFairWage(self.world) + self.profile.wage) / 2) - self.profile.wage)
         self.quitting_in = nil
-        self:setMood("pay_rise", "deactivate")
-        self.world.ui.bottom_panel:removeMessage(self)
+        self.hospital:removeMessage(self)
         self:increaseWage(amount)
         return
       end
@@ -200,10 +203,6 @@ function Staff:checkIfWaitedTooLong()
   end
 end
 
-function Staff:leaveAnnounce()
-  return
-end
-
 function Staff:isTiring()
   local tiring = true
 
@@ -246,7 +245,8 @@ function Staff:fire()
   if staff_window and staff_window.staff == self then
       staff_window:close()
   end
-  self.hospital:spendMoney(self.profile.wage, _S.transactions.severance .. ": " .. self.profile.name)
+  self.hospital:spendMoney(self.profile.wage, _S.transactions.severance .. ": "
+      .. self.profile:getFullName())
   self.world.ui:playSound("sack.wav")
   self:setMood("exit", "activate")
   self:setDynamicInfoText(_S.dynamic_info.staff.actions.fired)
@@ -255,7 +255,7 @@ function Staff:fire()
   self.hospital:changeReputation("kicked")
   self:despawn()
   self.hover_cursor = nil
-  self:leaveAnnounce()
+  self.hospital:announceStaffLeave(self)
   -- Unregister any build callbacks or messages.
   self:unregisterCallbacks()
   -- Update the staff management window if it is open.
@@ -331,7 +331,10 @@ end
 -- Function for increasing fatigue. Fatigue can be between 0 and 1,
 -- so amounts here should be appropriately small comma values.
 function Staff:tire(amount)
-  self:changeAttribute("fatigue", amount)
+  -- The no rest cheat overrides tiring effects
+  if not self.hospital.hosp_cheats:isCheatActive("no_rest_cheat") then
+    self:changeAttribute("fatigue", amount)
+  end
   self:updateDynamicInfo()
 end
 
@@ -351,8 +354,10 @@ function Staff:updateSpeed()
     level = 3
   end
   local room = self:getRoom()
-  if room and room.room_info.id == "training" then
+  if room and room.room_info.id == "training" and self:fulfillsCriterion("Doctor") then
     level = 1
+  elseif self.hospital and self.hospital.hosp_cheats:isCheatActive("no_rest_cheat") then
+    level = 3 -- Cheat makes everyone speedy
   elseif self.attributes["fatigue"] then
     if self.attributes["fatigue"] >= 0.8 then
       level = level - 2
@@ -387,14 +392,19 @@ function Staff:checkIfNeedRest()
     if self.waiting_for_staffroom then
       self:changeAttribute("happiness", -0.001)
     end
+
     local room = self:getRoom()
-    if (self.staffroom_needed and ((room and not room:getPatient()) or not room)) or
+    -- Nurses can leave a ward with patients remaining in their beds,
+    -- nurses in other rooms and other staff must leave after all patients exit
+    local may_leave = room and (room.room_info.id == "ward" or not room:getPatient())
+    if (self.staffroom_needed and (may_leave or not room)) or
         (room and self.going_to_staffroom) then
       if self:getCurrentAction().name ~= "walk" and self:getCurrentAction().name ~= "queue" then
         self.staffroom_needed = nil
         self:goToStaffRoom()
       end
     end
+
     -- Abort if waiting for a staffroom to be built, waiting for the patient to leave,
     -- already going to staffroom or being picked up
     if self.waiting_for_staffroom or self.staffroom_needed or
@@ -469,15 +479,15 @@ function Staff:adviseWrongPersonForThisRoom()
   local required = (room.room_info.maximum_staff or room.room_info.required_staff)
   if required then
     if required.Nurse then
-      self.world.ui.adviser:say(_A.staff_place_advice.only_nurses_in_room:format(room_name))
+      self.hospital:giveAdvice({ _A.staff_place_advice.only_nurses_in_room:format(room_name) })
     elseif required.Surgeon then
-      self.world.ui.adviser:say(_A.staff_place_advice.only_surgeons)
+      self.hospital:giveAdvice({ _A.staff_place_advice.only_surgeons })
     elseif required.Researcher then
-      self.world.ui.adviser:say(_A.staff_place_advice.only_researchers)
+      self.hospital:giveAdvice({ _A.staff_place_advice.only_researchers })
     elseif required.Psychiatrist then
-      self.world.ui.adviser:say(_A.staff_place_advice.only_psychiatrists)
+      self.hospital:giveAdvice({ _A.staff_place_advice.only_psychiatrists })
     else
-      self.world.ui.adviser:say(_A.staff_place_advice.only_doctors_in_room:format(room_name))
+      self.hospital:giveAdvice({ _A.staff_place_advice.only_doctors_in_room:format(room_name) })
     end
   end
 end
@@ -547,23 +557,29 @@ end
 
 -- Makes the staff member request a raise of 10%, or a wage exactly in the middle of their current and a fair one, whichever is more.
 function Staff:requestRaise()
-  -- Is this the first time a member of staff is requesting a raise?
-  -- Only show the help if the player is playing the campaign though
-  if self.hospital and not self.hospital.has_seen_pay_rise and tonumber(self.world.map.level_number) then
-    self.world.ui.adviser:say(_A.information.pay_rise)
-    self.hospital.has_seen_pay_rise = true
+  assert(self.hospital, "A staff member asked for a pay rise who doesn't belong to a hospital!")
+  local max_salary = self.world.map.level_config.payroll.MaxSalary
+  -- Never request a raise if already at max salary
+  if self.profile.wage >= max_salary then
+    self.timer_until_raise = nil
+    return
   end
   -- Check whether there is already a request for raise.
   if not self:isMoodActive("pay_rise") then
     local amount = math.floor(math.max(self.profile.wage * 1.1, (self.profile:getFairWage() + self.profile.wage) / 2) - self.profile.wage)
+    -- Don't ask over the salary cap
+    if self.profile.wage + amount > max_salary then
+      amount = max_salary - self.profile.wage
+    end
     -- At least for now, staff are timid, and only ask for raises 1/5th of the time
     if math.random(1, 5) ~= 1 or amount <= 0 then
       self.timer_until_raise = nil
-      return
+      return -- too timid
     end
+    -- The staff member can now successfully ask for a raise
+    self.hospital:makeRaiseRequest(amount, self)
     self.quitting_in = 25*30 -- Time until the staff members quits anyway
     self:setMood("pay_rise", "activate")
-    self.world.ui.bottom_panel:queueMessage("strike", amount, self)
   end
 end
 
@@ -571,31 +587,43 @@ end
 -- any request raise dialogs.
 --!param amount (integer) The amount, in game dollars per month, to increase
 -- the salary by.
+--!return (bool) Whether the wage actually increased
 function Staff:increaseWage(amount)
-  self.profile.wage = self.profile.wage + amount
-  self.world.ui:playSound("cashreg.wav")
-  if self.profile.wage > 2000 then -- What cap here?
-    self.profile.wage = 2000
-  else -- If the cap has been reached this member of staff won't get unhappy
-       -- ever again...
-    self:changeAttribute("happiness", 0.99)
-    self:setMood("pay_rise", "deactivate")
+  -- Are we already paid the maximum?
+  local max_salary = self.world.map.level_config.payroll.MaxSalary
+  local wage_raised = true
+  if self.profile.wage >= max_salary then
+    wage_raised = false -- Already at max salary
+  elseif self.profile.wage + amount > max_salary then
+    -- Maximum salary hit. The staff member will never be unhappy again
+    self.profile.wage = max_salary
+  else
+    -- Apply new salary
+    self.profile.wage = self.profile.wage + amount
   end
+  -- Reset
+  self:setMood("pay_rise", "deactivate")
+  self:changeAttribute("happiness", 0.99)
+  return wage_raised
 end
 
+--! Sets dynamic info text before the dynamic info update
+--!param text (string) the string to append
 function Staff:setDynamicInfoText(text)
   self.dynamic_text = text
   self:updateDynamicInfo()
 end
 
+--! Updates a staff member's dynamic info
 function Staff:updateDynamicInfo()
+  local dynamic_text = self.dynamic_text or ""
   local fatigue_text = _S.dynamic_info.staff.tiredness
   if not self.attributes["fatigue"] then
     fatigue_text = nil
   end
   self:setDynamicInfo('text', {
     self.profile.profession,
-    self.dynamic_text and self.dynamic_text or "",
+    dynamic_text,
     fatigue_text,
   })
   self:setDynamicInfo('progress', self.attributes["fatigue"])
@@ -669,6 +697,7 @@ function Staff:afterLoad(old, new)
     return
   end
 
+  self:updateDynamicInfo()
   Humanoid.afterLoad(self, old, new)
 end
 
@@ -700,4 +729,4 @@ function Staff:tostring()
 end
 
 -- Dummy callback for savegame compatibility
-local callbackNewRoom = --[[persistable:staff_build_staff_room_callback]] function(room) end
+local callbackNewRoom = --[[persistable:staff_build_staff_room_callback]] function() end
