@@ -53,10 +53,8 @@ If rating exceeds values, it will be capped as necessary
 --]]
 
 --[[ initialisation --]]
-corsixth.require("announcer")
 corsixth.require("utility")
 
-local AnnouncementPriority = _G["AnnouncementPriority"]
 
 --! A `Vip` who is in the hospital to evaluate the hospital and produce a report
 class "Vip" (Humanoid)
@@ -68,7 +66,7 @@ function Vip:Vip(...)
   self:Humanoid(...)
   self.hover_cursor = TheApp.gfx:loadMainCursor("default")
   self.action_string = ""
-  self.name=""
+  self.name = ""
   self.announced = false
   --First we should generate an initial VIP rating
   self.vip_rating = 12 - math.random(0,5)
@@ -92,7 +90,7 @@ function Vip:Vip(...)
 
 end
 
---[[--VIP while on premesis--]]
+--[[--VIP while on premises--]]
 function Vip:tickDay()
   -- for the vip
   if self.waiting then
@@ -225,62 +223,22 @@ end
 --[[--VIP has left--]]
 -- Called when the vip is out of the hospital grounds
 function Vip:onDestroy()
-  local message
-  -- First of all there's a special message if we're in free build mode.
   if self.world.free_build_mode then
     self.last_hospital:unconditionalChangeReputation(20)
-    message = {
-      {text = _S.fax.vip_visit_result.vip_remarked_name:format(self.name)},
-      {text = _S.fax.vip_visit_result.remarks.free_build[math.random(1, 3)]},
-      choices = {{text = _S.fax.vip_visit_result.close_text, choice = "close"}}
-    }
   elseif self.vip_rating <= 7 then
     self.last_hospital:receiveMoney(self.cash_reward, _S.transactions.vip_award)
     self.last_hospital:unconditionalChangeReputation(self.rep_reward)
     self.last_hospital.pleased_vips_ty = self.last_hospital.pleased_vips_ty + 1
-    message = {
-      {text = _S.fax.vip_visit_result.vip_remarked_name:format(self.name)},
-      {text = _S.fax.vip_visit_result.ordered_remarks[self.vip_message]},
-      {text = _S.fax.vip_visit_result.rep_boost},
-      {text = _S.fax.vip_visit_result.cash_grant:format(self.cash_reward)},
-      choices = {{text = _S.fax.vip_visit_result.close_text, choice = "close"}}
-    }
-  elseif self.vip_rating >=8 and self.vip_rating < 11 then
-    -- Dont tell player about any rep change in this range
-    self.last_hospital:unconditionalChangeReputation(self.rep_reward)
-    message = {
-      {text = _S.fax.vip_visit_result.vip_remarked_name:format(self.name)},
-      {text = _S.fax.vip_visit_result.ordered_remarks[self.vip_message]},
-      choices = {{text = _S.fax.vip_visit_result.close_text, choice = "close"}}
-    }
   else
     self.last_hospital:unconditionalChangeReputation(self.rep_reward)
-    message = {
-      {text = _S.fax.vip_visit_result.vip_remarked_name:format(self.name)},
-      {text = _S.fax.vip_visit_result.ordered_remarks[self.vip_message]},
-      {text = _S.fax.vip_visit_result.rep_loss},
-      choices = {{text = _S.fax.vip_visit_result.close_text, choice = "close"}}
-    }
   end
-  self.world.ui.bottom_panel:queueMessage("report", message, nil, 24 * 20, 1)
 
+  self.last_hospital:makeVipEndFax(self.vip_rating, self.name, self.cash_reward, self.vip_message)
   self.world:nextVip()
 
   return Humanoid.onDestroy(self)
 end
 
-function Vip:announce()
-  local announcements = {
-    "vip001.wav", "vip002.wav", "vip003.wav", "vip004.wav", "vip005.wav",
-  }   -- there is also vip008 which announces a man from the ministry
-  self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.High)
-  if self.hospital.num_vips < 1 then
-    self.world.ui.adviser:say(_A.information.initial_general_advice.first_VIP)
-  else
-    self.world.ui.adviser:say(_A.information.vip_arrived:format(self.name))
-  end
-end
-
 function Vip:setVIPRating()
   -- First do room code for later
   local count_rooms = 0
