@@ -20,10 +20,6 @@ SOFTWARE. --]]
 
 local TH = require("TH")
 
-corsixth.require("announcer")
-
-local AnnouncementPriority = _G["AnnouncementPriority"]
-
 --! An `Object` which needs occasional repair (to prevent explosion).
 class "Machine" (Object)
 
@@ -36,7 +32,7 @@ function Machine:Machine(hospital, object_type, x, y, direction, etc)
 
   if object_type.default_strength then
     -- Only for the main object. The slave doesn't need any strength
-    local progress = self.world.ui.hospital.research.research_progress[object_type]
+    local progress = self.hospital.research.research_progress[object_type]
     self.strength = progress.start_strength
   end
 
@@ -70,18 +66,6 @@ function Machine:isBreaking()
   return threshold < 4
 end
 
---! Announces a machine needing repair
---!param room The room of the machine
-function Machine:announceRepair(room)
-  local sound = room.room_info.handyman_call_sound
-  local earthquake = self.world.next_earthquake
-  self.world.ui:playAnnouncement("machwarn.wav", AnnouncementPriority.Critical)
-  -- If an earthquake is happening don't play the call sound to prevent spamming
-  if earthquake.active and earthquake.warning_timer == 0 then return end
-  -- TODO: Don't announce handyman call sound if there are no handymen
-  if sound then self.world.ui:playAnnouncement(sound, AnnouncementPriority.Critical) end
-end
-
 --! Set whether the smoke animation should be showing
 local function setSmoke(self, isSmoking)
   -- If turning smoke on for this machine
@@ -171,12 +155,13 @@ function Machine:machineUsed(room)
     if taskIndex == -1 then
       local call = self.world.dispatcher:callForRepair(self, true, false, true)
       self.hospital:addHandymanTask(self, "repairing", 2, self.tile_x, self.tile_y, call)
-      self:announceRepair(room)
+      self.hospital:announceRepair(room)
     else -- Otherwise the task is already queued. Increase the priority to above that of machines with at least 4 uses left
        -- Upgrades task from low (1) priority to high (2) priority
+       -- This does not lock the room, as happens when the task call starts at high priority
       if self.hospital:getHandymanTaskPriority(taskIndex, "repairing") == 1 then
         self.hospital:modifyHandymanTaskPriority(taskIndex, 2, "repairing")
-        self:announceRepair(room)
+        self.hospital:announceRepair(room)
       end
     end
   -- Else if repair is needed, but not urgently
@@ -252,7 +237,9 @@ function Machine:createHandymanActions(handyman)
   end
 
   local meander_loop_callback = --[[persistable:handyman_meander_repair_loop_callback]] function()
-    if not self.user then
+    -- Wait until the machine is not in use and not about to be used
+    local patient = self:getRoom():getPatient()
+    if not self.user and (not patient or patient:isLeaving()) then
       -- The machine is ready to be repaired.
       -- The following statement will finish the meander action in the handyman's
       -- action queue.
@@ -276,7 +263,11 @@ function Machine:createHandymanActions(handyman)
 end
 
 --! Replace this machine (make it pretend it's brand new)
-function Machine:machineReplaced()
+--!param cost (int) Cost to replace the machine
+function Machine:replaceMachine(cost)
+  -- Pay for the new machine
+  self.hospital:spendMoney(cost, _S.transactions.machine_replacement)
+
   -- Reset usage stats
   self.total_usage = 0
   self.times_used = 0
@@ -367,11 +358,20 @@ function Machine:updateDynamicInfo(only_update)
     self.total_usage = self.total_usage + 1
   end
   if self.strength then
-    self:setDynamicInfo("text", {
-      self.object_type.name,
-      _S.dynamic_info.object.strength:format(self.strength),
-      _S.dynamic_info.object.times_used:format(self.times_used),
-    })
+    if self.world.ui.app.config.new_machine_extra_info then
+      local hosp = self.world:getLocalPlayerHospital()
+      self:setDynamicInfo("text", {
+        self.object_type.name,
+        _S.dynamic_info.object.strength_extra_info:format(self.strength, hosp.research.research_progress[self.object_type].start_strength),
+        _S.dynamic_info.object.times_used:format(self.times_used),
+      })
+    else
+      self:setDynamicInfo("text", {
+        self.object_type.name,
+        _S.dynamic_info.object.strength:format(self.strength),
+        _S.dynamic_info.object.times_used:format(self.times_used),
+      })
+    end
   end
 end
 
@@ -417,6 +417,7 @@ function Machine:afterLoad(old, new)
       room.hospital:removeHandymanTask(taskIndex, "repairing")
     end
   end
+  self:updateDynamicInfo(true)
   return Object.afterLoad(self, old, new)
 end
 
@@ -488,5 +489,5 @@ function Machine:setHandymanRepairPosition(direction)
 end
 
 -- Dummy callbacks for savegame compatibility
-local callbackNewRoom = --[[persistable:machine_build_callback]] function(room) end
+local callbackNewRoom = --[[persistable:machine_build_callback]] function() end
 local repair_loop_callback = --[[persistable:handyman_repair_loop_callback]] function() end
