@@ -21,7 +21,6 @@ SOFTWARE. --]]
 local TH = require("TH")
 
 local pathsep = package.config:sub(1, 1)
-local ourpath = debug.getinfo(1, "S").source:sub(2, -17)
 
 --! Layer for loading (and subsequently caching) graphical resources.
 --! The Graphics class handles loading and caching of graphics resources.
@@ -57,8 +56,8 @@ local cursors_name = {
   banksummary = 44,
 }
 local cursors_palette = {
-  [36] = "bank01v.pal",
-  [44] = "stat01v.pal",
+  [36] = "Bank01V.pal",
+  [44] = "Stat01V.pal",
 }
 
 function Graphics:Graphics(app)
@@ -97,7 +96,7 @@ function Graphics:Graphics(app)
   if self.app.config.use_new_graphics then
     -- Check if the config specifies a place to look for graphics in.
     -- Otherwise check in the default "Graphics" folder.
-    graphics_folder = self.app.config.new_graphics_folder or ourpath .. "Graphics"
+    graphics_folder = self.app.config.new_graphics_folder or self.app:getFullPath("Graphics", true)
     if graphics_folder:sub(-1) ~= pathsep then
       graphics_folder = graphics_folder .. pathsep
     end
@@ -120,23 +119,37 @@ end
 --! Tries to load the font file given in the config file as unicode_font.
 --! If it is not found it tries to find one in the operating system.
 function Graphics:loadFontFile()
+  local lfs = require("lfs")
+  local function check(path) return path and lfs.attributes(path, "mode") == "file" end
   -- Load the Unicode font, if there is one specified.
-  local font_file = self.app.config.unicode_font
-  if not font_file then
-    -- Try a font which commonly comes with the operating system.
-    local windir = os.getenv("WINDIR")
-    if windir and windir ~= "" then
-      font_file = windir .. pathsep .. "Fonts" .. pathsep .. "ARIALUNI.TTF"
-    elseif self.app.os == "macos" then
-      font_file = "/Library/Fonts/Arial Unicode.ttf"
-    else
-      font_file = "/usr/share/fonts/truetype/arphic/uming.ttc"
-    end
+  local config_path = self.app.config.unicode_font
+  -- Try a font that commonly comes with the operating system.
+  local os_path, font_file
+  local windir = os.getenv("WINDIR")
+  if windir and windir ~= "" then
+    os_path = windir .. pathsep .. "Fonts" .. pathsep .. "ARIALUNI.TTF"
+  elseif self.app.os == "macos" then
+    os_path = "/Library/Fonts/Arial Unicode.ttf"
+  else
+    os_path = "/usr/share/fonts/truetype/arphic/uming.ttc"
   end
-  font_file = font_file and io.open(font_file, "rb")
-  if font_file then
-    self.ttf_font_data = font_file:read"*a"
-    font_file:close()
+  if check(config_path) then font_file = config_path
+  elseif check(os_path) then
+    font_file = os_path
+    print("Configured unicode font not found, using " .. font_file .. " instead.")
+    print("This will be written to the config file.")
+  elseif config_path ~= nil then
+    print("Configured unicode font not found, no fallback available.")
+    return
+  end
+  local font = font_file and io.open(font_file, "rb")
+  if font then
+    self.ttf_font_data = font:read("*a")
+    font:close()
+    if self.ttf_font_data and self.app.config.unicode_font ~= font_file then
+      self.app.config.unicode_font = font_file
+      self.app:saveConfig()
+    end
   end
 end
 
@@ -145,8 +158,7 @@ function Graphics:loadMainCursor(id)
     id = cursors_name[id]
   end
   if id > 20 then -- SPointer cursors
-    local cursor_palette = self:loadPalette("QData", cursors_palette[id])
-    cursor_palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
+    local cursor_palette = self:loadPalette("QData", cursors_palette[id], true)
     return self:loadCursor(self:loadSpriteTable("QData", "SPointer", false, cursor_palette), id - 20)
   else
     return self:loadCursor(self:loadSpriteTable("Data", "MPointer"), id)
@@ -214,9 +226,20 @@ local function makeGreyscaleGhost(pal)
   return table.concat(remap, "", 0, 255)
 end
 
-function Graphics:loadPalette(dir, name)
+--! Load a palette file
+--!param dir (string) The directory of the palette relative to the HOSPITAL directory
+--!param name (string) The name of the palette file
+--!param transparent_255 (boolean) Whether the 255th entry in the palette should be transparent
+--!return (palette, string) The palette and a string representing the palette converted to greyscale
+function Graphics:loadPalette(dir, name, transparent_255)
   name = name or "MPalette.dat"
+
   if self.cache.palette[name] then
+    local li = self.load_info[self.cache.palette[name]]
+    if li and li[5] ~= transparent_255 then
+      print("Warning: palette " .. name .. " requested with different flags than stored")
+    end
+
     return self.cache.palette[name],
       self.cache.palette_greyscale_ghost[name]
   end
@@ -224,9 +247,12 @@ function Graphics:loadPalette(dir, name)
   local data = self.app:readDataFile(dir or "Data", name)
   local palette = TH.palette()
   palette:load(data)
+  if transparent_255 then
+    palette:setEntry(255, 0xFF, 0x00, 0xFF)
+  end
   self.cache.palette_greyscale_ghost[name] = makeGreyscaleGhost(data)
   self.cache.palette[name] = palette
-  self.load_info[palette] = {self.loadPalette, self, dir, name}
+  self.load_info[palette] = {self.loadPalette, self, dir, name, transparent_255}
   return palette, self.cache.palette_greyscale_ghost[name]
 end
 
@@ -240,7 +266,16 @@ function Graphics:loadGhost(dir, name, index)
   return cached:sub(index * 256 + 1, index * 256 + 256)
 end
 
-function Graphics:loadRaw(name, width, height, dir, paldir, pal)
+--! Load a bitmap from a dat file and palette
+--!
+--!param name (string) The file name of the bitmap without the .dat extension
+--!param width (int) The width of the bitmap. Defaults to 640
+--!param height (int) The height of the bitmap. Defaults to 480
+--!param dir (string) The directory of the bitmap. Defaults to QData
+--!param paldir (string) The directory of the palette.
+--!param pal (string) The name of the palette
+--!param transparent_255 (boolean) Whether the 255th entry of the palette should be transparent
+function Graphics:loadRaw(name, width, height, dir, paldir, pal, transparent_255)
   if self.cache.raw[name] then
     return self.cache.raw[name]
   end
@@ -248,16 +283,13 @@ function Graphics:loadRaw(name, width, height, dir, paldir, pal)
   width = width or 640
   height = height or 480
   dir = dir or "QData"
+  paldir = paldir or dir
+  pal = pal or (name .. ".pal")
   local data = self.app:readDataFile(dir, name .. ".dat")
   data = data:sub(1, width * height)
 
   local bitmap = TH.bitmap()
-  local palette
-  if pal and paldir then
-    palette = self:loadPalette(paldir, pal)
-  else
-    palette = self:loadPalette(dir, name .. ".pal")
-  end
+  local palette = self:loadPalette(paldir, pal, transparent_255)
   bitmap:setPalette(palette)
   assert(bitmap:load(data, width, self.target))
 
@@ -270,7 +302,7 @@ function Graphics:loadRaw(name, width, height, dir, paldir, pal)
   self.reload_functions[bitmap] = bitmap_reloader
 
   self.cache.raw[name] = bitmap
-  self.load_info[bitmap] = {self.loadRaw, self, name, width, height, dir, paldir, pal}
+  self.load_info[bitmap] = {self.loadRaw, self, name, width, height, dir, paldir, pal, transparent_255}
   return bitmap
 end
 
@@ -416,7 +448,7 @@ function Graphics:loadFont(sprite_table, x_sep, y_sep, ...)
 
   local use_bitmap_font = true
   -- Force bitmap font for the moneybar (Font05V)
-  if not sprite_table:isVisible(46) or load_font == "Font05V" then -- uppercase M
+  if not sprite_table:isVisible(46) or load_font == "Font05V" then -- luacheck: ignore 542
     -- The font doesn't contain an uppercase M, so (in all likelihood) is used
     -- for drawing special symbols rather than text, so the original bitmap
     -- font should be used.
@@ -451,7 +483,7 @@ function Graphics:loadAnimations(dir, prefix)
     if not file then
       return nil, err
     end
-    local data = file:read"*a"
+    local data = file:read("*a")
     file:close()
     return data
   end
