@@ -20,8 +20,6 @@ SOFTWARE. --]]
 
 corsixth.require("announcer")
 
-local AnnouncementPriority = _G["AnnouncementPriority"]
-
 class "Hospital"
 
 ---@type Hospital
@@ -58,7 +56,6 @@ function Hospital:Hospital(world, avail_rooms, name)
   -- Initial value: hospital tile count * tile value + 20000
   self.value = world.map:getParcelPrice(self:getPlayerIndex()) + 20000
 
-  -- TODO: With the fame/shame screen and scoring comes salary.
   self.player_salary = 10000
   self.salary_offer = 0
 
@@ -112,13 +109,22 @@ function Hospital:Hospital(world, avail_rooms, name)
     heating_broke = false -- (bool) Whether the heating system is broken down currently.
   }
 
+  -- Number of tile objects of each category in the hospital.
+  self.tile_object_counts = {
+    extinguisher = 0,
+    radiator = 0,
+    plant = 0,
+    reception_desk = 0,
+    bench = 0,
+    general = 0,
+  }
+
   self.num_visitors = 0
   self.num_deaths = 0
   self.num_deaths_this_year = 0
   self.num_cured = 0
   self.not_cured = 0
   self.num_explosions = 0
-  self.announce_vip = 0
   self.vip_declined = 0
   self.num_vips = 0 -- used to check if it's the user's first vip
   self.percentage_cured = 0
@@ -126,8 +132,6 @@ function Hospital:Hospital(world, avail_rooms, name)
   self.population = 0.25 -- TODO: Percentage showing how much of
   -- the total population that goes to the player's hospital,
   -- used for one of the goals. Change when competitors are there.
-  -- Since there are none right now the player's hospital always get
-  -- 50 % of all patients as soon as gbv.AllocDelay has expired.
 
   -- Statistics used in the graph dialog. Each entry is the month, inside it
   -- is "money in", "money out", wages, balance, visitors, cures, deaths, reputation
@@ -170,6 +174,9 @@ function Hospital:Hospital(world, avail_rooms, name)
   self.policies = {}
   self.discovered_diseases = {} -- a list
 
+  -- Create a cheats instance for each hospital. AIHospitals don't know how to cheat (we hope!)
+  self.hosp_cheats = Cheats(self)
+
   -- Make a table containing available rooms for the level, and its discovery status (room, discovery_status)
   -- Index is the room's id e.g. gp
   self.room_discoveries = {}
@@ -270,16 +277,6 @@ function Hospital:isRoomDiscovered(room_id)
   return self.room_discoveries[room_id].is_discovered
 end
 
---! Give the user possibly a message about a cured patient.
-function Hospital:msgCured()
-  -- Nothing to do, override in a derived class.
-end
-
---! Give the user possibly a message about a dead patient.
-function Hospital:msgKilled()
-  -- Nothing to do, override in a derived class.
-end
-
 --! Update the loaded game with version 'old' to the version 'new'.
 --!param old Version of the loaded game.
 --!param new Version of the code being executed.
@@ -647,6 +644,43 @@ function Hospital:afterLoad(old, new)
   if old < 155 then
     self.overdraft_interest_rate = self.interest_rate + 0.02
   end
+  if old < 159 then
+    self.receptionist_msg = nil
+  end
+
+  if old < 167 then
+    -- Count tile objects of this hospital from scratch.
+    self.tile_object_counts = {
+      extinguisher = 0,
+      radiator = 0,
+      plant = 0,
+      reception_desk = 0,
+      bench = 0,
+      general = 0,
+    }
+    local width, height = self.world.map.th:size()
+    for x = 1, width do
+      for y = 1, height do
+        local objects = self.world:getObjects(x, y)
+        if objects then
+          for _, object in ipairs(objects) do
+            if object.hospital == self then
+              self:addTileObject(object.object_type.count_category)
+            end
+          end
+        end
+      end
+    end
+  end
+
+  if old < 169 then
+    -- AIHospital previously did not have a cheats instance, now it should
+    if not self.hosp_cheats then self.hosp_cheats = Cheats(self) end
+    -- Move toggle cheat triggers to Cheats
+    self.hosp_cheats.active_cheats = {}
+    self.hosp_cheats.active_cheats["spawn_rate_cheat"] = self.spawn_rate_cheat
+    self.spawn_rate_cheat = nil
+  end
 
   -- Update other objects in the hospital (added in version 106).
   if self.epidemic then self.epidemic.afterLoad(old, new) end
@@ -722,7 +756,7 @@ end
 
 function Hospital:purchasePlot(plot_number)
   local map = self.world.map
-  if map.th:isParcelPurchasable(plot_number, self:getPlayerIndex()) and not self.world.ui.transparent_walls then
+  if map.th:isParcelPurchasable(plot_number, self:getPlayerIndex()) then
     local cost = not self.world.free_build_mode and map:getParcelPrice(plot_number) or 0
     if cost <= self.balance then
       self.world:setPlotOwner(plot_number, self:getPlayerIndex())
@@ -732,9 +766,8 @@ function Hospital:purchasePlot(plot_number)
       self:spendMoney(cost, _S.transactions.buy_land, cost)
       return true
     else
-    -- Give visual warning that player doesn't have enough $ to build
-    -- Let the message remain unitl cancelled by the player as it is being displayed behind the town map
-      self.world.ui.adviser:say(_A.warnings.cannot_afford_2, true, true)
+      -- Give visual warning that player doesn't have enough $ to build
+      self:adviseCannotAffordPlot()
     end
   end
   return false
@@ -774,22 +807,6 @@ function Hospital:isInHospital(x, y)
   local flags = self.world.map.th:getCellFlags(x, y)
   return flags.hospital and flags.owner == self:getPlayerIndex()
 end
-function Hospital:coldWarning()
-  local announcements = {
-    "sorry002.wav", "sorry004.wav",
-  }
-  if announcements and self:isPlayerHospital() then
-    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Normal)
-  end
-end
-function Hospital:hotWarning()
-  local announcements = {
-    "sorry003.wav", "sorry004.wav",
-  }
-  if announcements and self:isPlayerHospital() then
-    self.world.ui:playAnnouncement(announcements[math.random(1, #announcements)], AnnouncementPriority.Normal)
-  end
-end
 
 --! Decide how many days the hospital functions within specification.
 --!return (int) Number of disaster-free days in the hospital.
@@ -818,16 +835,8 @@ function Hospital:boilerBreakdown(broken_heat)
   heat_vars.boiler_repair_count = math.random(10, 30)
   heat_vars.heating_broke = true
 
-  -- Only show the message when relevant to the local player's hospital.
-  if self:isPlayerHospital() then
-    if heat_vars.radiator_heat == 0 then
-      self.world.ui.adviser:say(_A.boiler_issue.minimum_heat)
-      self:coldWarning()
-    else
-      self.world.ui.adviser:say(_A.boiler_issue.maximum_heat)
-      self:hotWarning()
-    end
-  end
+  -- Warn the player of the boiler's breakdown
+  self:adviseBoilerBreakdown(broken_heat)
 end
 
 --! Boiler broke down and work is done to get it fixed.
@@ -851,9 +860,9 @@ function Hospital:_fixBoiler()
     -- It's fixed, restore previous settings.
     heat_vars.radiator_heat = heat_vars.saved_radiator_heat
     heat_vars.heating_broke = false
-    if num_radiators > 0 and self:isPlayerHospital() then
+    if num_radiators > 0 then
       -- Only tell the player about fix if there is at least one radiator.
-      self.world.ui.adviser:say(_A.boiler_issue.resolved)
+      self:giveAdvice({_A.boiler_issue.resolved})
     end
   end
 end
@@ -923,21 +932,6 @@ function Hospital:onEndDay()
     end
   end
 
-  -- check if we still have to announce VIP visit
-  if self.announce_vip > 0 then
-    -- check if the VIP is in the building yet
-    for _, e in ipairs(self.world.entities) do
-      if e.humanoid_class == "VIP" and e.announced == false then
-        if self:isInHospital(e.tile_x, e.tile_y) and self:isPlayerHospital() then
-          -- play VIP arrival sound and show tooltips
-          e:announce()
-          e.announced = true
-          self.announce_vip = self.announce_vip - 1
-        end
-      end
-    end
-  end
-
   self:_fixBoiler() -- Boiler always needs work (especially if broken).
 
   -- Do we have a disaster?
@@ -1056,11 +1050,10 @@ end
 --! Collect the reception desks in the hospital.
 --!return (list) The reception desks in the hospital.
 function Hospital:findReceptionDesks()
-  -- TODO Breaks in multiplayer mode.
   local reception_desks = {}
   for _, obj_list in pairs(self.world.objects) do
     for _, obj in ipairs(obj_list) do
-      if obj.object_type.id == "reception_desk" then
+      if obj.hospital == self and obj.object_type.id == "reception_desk" then
         reception_desks[#reception_desks + 1] = obj
       end
     end
@@ -1078,9 +1071,8 @@ function Hospital:onEndYear()
   self:unconditionalChangeReputation(0) -- Reset self.has_impressive_reputation
 
   -- On third year of level 3 there is the large increase to salary
-  -- this will replicate that. I have still to check other levels above 5 to
+  -- this will replicate that. TODO: check other levels above 5 to
   -- see if there are other large increases.
-  -- TODO Hall of fame and shame
   if self.world:date():year() == 3 and self.world.map.level_number == 3 then
     -- adds the extra to salary in level 3 year 3
     self.player_salary = self.player_salary + math.random(8000,20000)
@@ -1113,51 +1105,7 @@ function Hospital:createEmergency(emergency)
     end
 
     self.emergency = emergency
-    -- The last room in the list of treatment rooms is considered when checking for availability.
-    -- It works for all original diseases, but if we introduce new multiple room diseases it might break.
-    -- TODO: Make it work for all kinds of lists of treatment rooms.
-    -- TODO: Change to make use of Hospital:checkDiseaseRequirements
-    local no_rooms = #emergency.disease.treatment_rooms
-    local room_name, required_staff, staff_name =
-      self.world:getRoomNameAndRequiredStaffName(emergency.disease.treatment_rooms[no_rooms])
-
-    local staff_available = self:countStaffOfCategory(required_staff) > 0
-    -- Check so that all rooms in the list are available
-    if self:countRoomOfType(emergency.disease.treatment_rooms[no_rooms], 1) > 0 then
-      room_name = nil
-    end
-
-    local casebook = self.disease_casebook[random_disease.id]
-    local added_info = casebook.drug and
-        _S.fax.emergency.cure_possible_drug_name_efficiency:format(emergency.disease.name, casebook.cure_effectiveness)
-        or _S.fax.emergency.cure_possible
-    if room_name then
-      if staff_available then
-        added_info = _S.fax.emergency.cure_not_possible_build:format(room_name) .. "."
-      else
-        added_info = _S.fax.emergency.cure_not_possible_build_and_employ:format(room_name, staff_name) .. "."
-      end
-    elseif not staff_available then
-      added_info = _S.fax.emergency.cure_not_possible_employ:format(staff_name) .. "."
-    end
-
-    local one_or_many_victims_msg
-    if emergency.victims == 1 then
-      one_or_many_victims_msg = _S.fax.emergency.num_disease_singular:format(emergency.disease.name)
-    else
-      one_or_many_victims_msg = _S.fax.emergency.num_disease:format(emergency.victims, emergency.disease.name)
-    end
-    local message = {
-      {text = _S.fax.emergency.location:format(_S.fax.emergency.locations[math.random(1,9)])},
-      {text = one_or_many_victims_msg },
-      {text = added_info},
-      {text = self.world.free_build_mode and _S.fax.emergency.free_build or _S.fax.emergency.bonus:format(emergency.bonus*emergency.victims)},
-      choices = {
-        {text = _S.fax.emergency.choices.accept, choice = "accept_emergency"},
-        {text = _S.fax.emergency.choices.refuse, choice = "refuse_emergency"},
-      },
-    }
-    self.world.ui.bottom_panel:queueMessage("emergency", message, nil, Date.hoursPerDay() * 16, 2) -- automatically refuse after 16 days
+    self:makeEmergencyStartFax()
     return -- successfully created
   end
   return "no heliport"
@@ -1180,15 +1128,7 @@ function Hospital:resolveEmergency()
   if emergency_success then
     earned = emer.bonus * rescued_patients
   end
-  local message = {
-    {text = _S.fax.emergency_result.saved_people
-      :format(rescued_patients, total)},
-    {text = self.world.free_build_mode and "" or _S.fax.emergency_result.earned_money:format(max_bonus, earned)},
-    choices = {
-      {text = _S.fax.emergency_result.close_text, choice = "close"},
-    },
-  }
-  self.world.ui.bottom_panel:queueMessage("report", message, nil, 24*25, 1)
+  self:makeEmergencyEndFax(rescued_patients, total, max_bonus, earned)
   if emergency_success then -- Reputation increased
     self:changeReputation("emergency_success", emer.disease)
     self:receiveMoney(earned, _S.transactions.emergency_bonus)
@@ -1212,25 +1152,13 @@ function Hospital:checkEmergencyOver()
   end
 end
 
--- Creates VIP and sends a FAX to query the user.
-function Hospital:createVip()
-  local vipName =  _S.vip_names[math.random(1,10)]
-  local message = {
-    {text = _S.fax.vip_visit_query.vip_name:format(vipName)},
-    choices = {{text = _S.fax.vip_visit_query.choices.invite, choice = "accept_vip", additionalInfo = {name=vipName}},
-               {text = _S.fax.vip_visit_query.choices.refuse, choice = "refuse_vip", additionalInfo = {name=vipName}}}
-  }
-  -- auto-refuse after 20 days
-  self.world.ui.bottom_panel:queueMessage("personality", message, nil, 24*20, 2)
-end
-
 --[[ Creates a new epidemic by creating a new contagious patient with
  a random disease - this is NOT typically how epidemics are started (mainly for cheat use)
  see @Hospital:determineIfContagious() to see how epidemics are typically started]]
 function Hospital:spawnContagiousPatient()
   --[[ Gets the available non-visual disease in the current world
     @return non_visuals (table) table of available non-visual diseases]]
-  local function get_avaliable_contagious_diseases()
+  local function get_available_contagious_diseases()
     local contagious = {}
     for _, disease in ipairs(self.world.available_diseases) do
       if disease.contagious then
@@ -1242,7 +1170,7 @@ function Hospital:spawnContagiousPatient()
 
   if self:hasStaffedDesk() then
     local patient = self.world:newEntity("Patient", 2)
-    local contagious_diseases = get_avaliable_contagious_diseases()
+    local contagious_diseases = get_available_contagious_diseases()
     if #contagious_diseases > 0 then
       local disease = contagious_diseases[math.random(1,#contagious_diseases)]
       patient:setDisease(disease)
@@ -1606,7 +1534,8 @@ end
 function Hospital:addStaff(staff)
   self.staff[#self.staff + 1] = staff
   -- Cost of hiring staff:
-  self:spendMoney(staff.profile.wage, _S.transactions.hire_staff .. ": " .. staff.profile.name)
+  self:spendMoney(staff.profile.wage, _S.transactions.hire_staff .. ": "
+    .. staff.profile:getFullName())
   self:notifyOfStaffChange(staff)
 end
 
@@ -1673,39 +1602,54 @@ function Hospital:countRoomOfType(type, max_count)
   return result
 end
 
+--! Update tile object counts for adding an object.
+--! See also 'Hospital:removeTileObject'
+--!param object_category Category of the tile object.
+function Hospital:addTileObject(object_category)
+  if object_category then
+    local current_count = self.tile_object_counts[object_category]
+    self.tile_object_counts[object_category] = current_count + 1
+  end
+end
+
+--! Update tile object counts for removing an object.
+--! See also 'Hospital:addTileObject'
+--!param object_category Category of the tile object.
+function Hospital:removeTileObject(object_category)
+  if object_category then
+    local current_count = self.tile_object_counts[object_category]
+    self.tile_object_counts[object_category] = current_count - 1
+  end
+end
+
 --! Get the number of reception desks in the hospital.
 --!return (int) Number of reception desks in the hospital.
 function Hospital:countReceptionDesks()
-  -- TODO Breaks in multiplayer mode.
-  return self.world.object_counts["reception_desk"]
+  return self.tile_object_counts["reception_desk"]
 end
 
 --! Get the number of radiators in the hospital.
 --!return (int) Number of radiators in the hospital.
 function Hospital:countRadiators()
-  -- TODO Breaks in multiplayer mode.
-  return self.world.object_counts["radiator"]
+  return self.tile_object_counts["radiator"]
 end
 
 --! Get the number of plants in the hospital.
 --!return (int) Number of plants in the hospital.
 function Hospital:countPlants()
-  -- TODO Breaks in multiplayer mode.
-  return self.world.object_counts["plant"]
+  return self.tile_object_counts["plant"]
 end
 
 --! Get the number of fire extinguishers in the hospital.
 --!return (int) Number of fire extinguishers in the hospital.
 function Hospital:countFireExtinguishers()
-  -- TODO Breaks in multiplayer mode.
-  return self.world.object_counts["extinguisher"]
+  return self.tile_object_counts["extinguisher"]
 end
 
 --! Get the number of general objects in the hospital.
 --!return (int) Number of general objects in the hospital.
 function Hospital:countGeneralObjects()
-  -- TODO Breaks in multiplayer mode.
-  return self.world.object_counts["general"]
+  return self.tile_object_counts["general"]
 end
 
 --! A new object has been placed in the hospital.
@@ -1718,9 +1662,11 @@ function Hospital:objectPlaced(entity, id)
     local w, h = self.world.map.th:size()
     for tx = math.max(1, entity.tile_x - notify_distance), math.min(w, entity.tile_x + notify_distance) do
       for ty = math.max(1, entity.tile_y - notify_distance), math.min(h, entity.tile_y + notify_distance) do
-        for _, patient in ipairs(self.world.entity_map:getHumanoidsAtCoordinate(tx, ty)) do
-          if class.is(patient, Patient) then
-            patient:notifyNewObject(id)
+        if self:isInHospital(tx, ty) then -- Only patients of this hospital should be warned.
+          for _, patient in ipairs(self.world.entity_map:getHumanoidsAtCoordinate(tx, ty)) do
+            if class.is(patient, Patient) then
+              patient:notifyNewObject(id)
+            end
           end
         end
       end
@@ -1734,7 +1680,7 @@ function Hospital:objectPlaced(entity, id)
   end
 
   if id == "plant" then
-    self:msgPlant()
+    self:advisePlants(true)
     return
   end
 
@@ -1744,22 +1690,6 @@ function Hospital:objectPlaced(entity, id)
   end
 end
 
---! Give advice to the user about having bought a reception desk.
-function Hospital:msgReceptionDesk()
-  -- Nothing to do, override in a sub-class.
-end
-
---! Give advice to the user about maintenance of plants.
-function Hospital:msgPlant()
-  -- Nothing to do, override in a sub-class.
-end
-
---! Show the 'Gates to hell' animation.
---!param _entity (Entity) Gates to hell.
-function Hospital:showGatesToHell(_entity)
-  -- Nothing to do, override in a sub-class.
-end
-
 --! Remove the first entry with a given value from a table.
 --! Only works reliably for lists.
 --!param t Table to search for the value, and update.
@@ -2365,17 +2295,17 @@ function Hospital:computePriceLevelImpact(patient, casebook)
 
   if price_distortion < self.under_priced_threshold then
     if math.random(1, 100) == 1 then
-      self.world.ui.adviser:say(_A.warnings.low_prices:format(casebook.disease.name))
+      self:advisePriceLevelImpact("under", casebook.disease.name)
       self:changeReputation("under_priced")
     end
   elseif price_distortion > self.over_priced_threshold then
     if math.random(1, 100) == 1 then
-      self.world.ui.adviser:say(_A.warnings.high_prices:format(casebook.disease.name))
+      self:advisePriceLevelImpact("over", casebook.disease.name)
       self:changeReputation("over_priced")
     end
   elseif math.abs(price_distortion) <= 0.15 and math.random(1, 200) == 1 then
     -- When prices are well adjusted (i.e. abs(price distortion) <= 0.15)
-    self.world.ui.adviser:say(_A.warnings.fair_prices:format(casebook.disease.name))
+    self:advisePriceLevelImpact("fair", casebook.disease.name)
   end
 end
 
@@ -2434,3 +2364,95 @@ function Hospital:getRandomBusyRoom()
   local chosen_room = long_queue_rooms[math.random(1, #long_queue_rooms)]
   if #chosen_room.door.queue >= busy_threshold then return chosen_room end
 end
+
+---- Stubs section - these functions have nothing to do here, are overridden in a derived class.
+
+--! Give advice to the user about the need to buy the first reception desk.
+function Hospital:msgNeedFirstReceptionDesk()
+end
+
+--! Give advice to the user about having bought a reception desk.
+function Hospital:msgReceptionDesk()
+end
+
+--! Give advice about having more desks.
+function Hospital:msgMultiReceptionDesks()
+end
+
+--! Give advice to the user about maintenance of plants.
+function Hospital:advisePlants()
+end
+
+--! Show the 'Gates to hell' animation.
+function Hospital:showGatesToHell(entity)
+end
+
+--! Advises the player.
+function Hospital:giveAdvice(msgs, rnd_frac, stay_up)
+end
+
+--! Give the user possibly a message about a cured patient.
+function Hospital:msgCured()
+end
+
+--! Give the user possibly a message about a dead patient.
+function Hospital:msgKilled()
+end
+
+function Hospital:adviseDiscoverDisease(disease)
+end
+
+--! Select a relevant message to be displayed to the user
+function Hospital:adviseBoilerBreakdown()
+end
+
+--! Announces a machine needing repair
+function Hospital:announceRepair(room)
+end
+
+function Hospital:onSpawnVIP()
+end
+
+--! Give visual warning that player doesn't have enough $ to build
+function Hospital:adviseCannotAffordPlot()
+end
+
+--! Tell the player, through the advisor, about the impact of casebook prices
+function Hospital:advisePriceLevelImpact(judgment, name)
+end
+
+--! Makes the raise request for a staff member
+function Hospital:makeRaiseRequest(amount, staff)
+end
+
+--! Announce to the player that a staff member is leaving the hospital
+function Hospital:announceStaffLeave(staff)
+end
+
+--! Makes the fax requesting player action for the patient who needs a diagnosis room
+function Hospital:makeNoTreatmentRoomFax(patient)
+end
+
+--! Makes the fax requesting player action for the patient who needs a diagnosis room
+function Hospital:makeNoDiagnosisRoomFax(patient)
+end
+
+--! Makes the fax at the start of an emergency
+function Hospital:makeEmergencyStartFax()
+end
+
+--! Makes the fax at the end of an emergency
+function Hospital:makeEmergencyEndFax(rescued_patients, total, max_bonus, earned)
+end
+
+--! Makes the fax which may spawn a VIP
+function Hospital:createVip()
+end
+
+--! Remove any message (fax) relating to this humanoid
+function Hospital:removeMessage(humanoid)
+end
+
+--! Called when the vip is out of the hospital grounds
+function Hospital:makeVipEndFax(vip_rating, name, cash_reward, vip_message)
+end
