@@ -24,8 +24,8 @@ Inherit("original_strings", 0)
 --Note: All strings should use a single space after full-stops. Only exception is level descriptions.
 -------------------------------  OVERRIDE  ----------------------------------
 adviser.warnings.money_low = "Your money is getting low!" -- Funny. Exists in German translation, but not existent in english?
--- TODO: tooltip.graphs.reputation -- this tooltip talks about hospital value. Actually it should say reputation.
--- TODO: tooltip.status.close -- it's called status window, not overview window.
+tooltip.graphs.reputation = "Toggle reputation" -- Incorrectly said "toggle hospital value"
+tooltip.status.close = "Close status window" -- Incorrectly said "close overview window"
 
 -- tooltip.staff_list.next_person, prev_person is rather next/prev page (also in german, maybe more languages?)
 tooltip.staff_list.next_person = "Show next page"
@@ -79,7 +79,7 @@ pay_rise.regular[1] = "I'm totally frazzled. I need a good rest, plus a rise of
 
 -- Level description overrides where there are typos. Note: This is the only portion of the game that SHOULD use double space after fullstops etc.
 introduction_texts.level17 = " Last word of warning - keep a keen eye on your Reputation - this is what will attract the patients from far and wide to your establishment.  If you don't kill too many people and keep them reasonably happy you shouldn't have too much trouble on this level!// You're on you own now.  Good luck and all that."
-introduction_texts.level11 = " You've been given the chance to build the ultimate in hospitals.  This is an exceedingly prestigious area, and the Ministry would like to see the best possible hospital.  We'll be expecting you to make big money, have a superbly high reputation and cover every possible eventuality.  It's an important job, this.  You'll have to be something special to pull it off.  Note, too, that there have sightings of UFOs in the area.  Make sure your staff are prepared for some unexpected visitors.  Your hospital will need to be worth $240,000, you'll need $500,000 in the bank, and your reputation will need to be 700."
+introduction_texts.level11 = " You've been given the chance to build the ultimate in hospitals.  This is an exceedingly prestigious area, and the Ministry would like to see the best possible hospital.  We'll be expecting you to make big money, have a superbly high reputation and cover every possible eventuality.  It's an important job, this.  You'll have to be something special to pull it off.  Note, too, that there have been sightings of UFOs in the area.  Make sure your staff are prepared for some unexpected visitors.  Your hospital will need to be worth $240,000, you'll need $500,000 in the bank, and your reputation will need to be 700."
 introduction_texts.level9 = " Having filled the Ministry's bank account and paid for a new limousine for the Minister himself, you can now get back to creating a caring, well-run hospital for the benefit of the unwell and needy.  You can expect a lot of different problems to crop up here.  If you have enough well-trained staff and rooms, you should have all the angles covered.  Your hospital will need to be worth $200,000, and you'll need $400,000 in the bank.  Any less and you won't be able to finish the level."
 introduction_texts.level16 = " Once you have diagnosed some of the patients you will need to build treatment facilities and clinics to cure them - a good one to start off with is the Pharmacy.  You'll also need a Nurse to dispense the various drugs in the Pharmacy."
 introduction_texts.level10 = " As well as covering all the illnesses which crop up in this neck of the woods, the Ministry request that you spend some time concentrating on the efficiency of your drugs.  There have been some complaints from Ofsick, the Health Watchdog, so in order to look good you must make sure all your drugs are extremely efficient.  Also, make sure your hospital is above reproach as well. Keep those deaths down.  As a hint, you might like to keep space free for a Jelly Vat.  Develop all your drugs to at least 80 per cent efficiency, get a reputation of 650 and stash $500,000 in the bank to win."
@@ -219,10 +219,10 @@ menu_debug_overlay = {
   parcel                      = "  PARCEL  ",
 }
 menu_player_count = {
-	players_1 = "  1 PLAYER  ",
-	players_2 = "  2 PLAYERS  ",
-	players_3 = "  3 PLAYERS  ",
-	players_4 = "  4 PLAYERS  ",
+  players_1 = "  1 PLAYER  ",
+  players_2 = "  2 PLAYERS  ",
+  players_3 = "  3 PLAYERS  ",
+  players_4 = "  4 PLAYERS  ",
 }
 adviser = {
   room_forbidden_non_reachable_parts = "Placing the room in this location would result in parts of the hospital not being reachable.",
@@ -260,6 +260,11 @@ adviser = {
     th_cheat = "Congratulations, you have unlocked cheats!",
     roujin_on_cheat = "Roujin's challenge activated! Good luck in the coming months...",
     roujin_off_cheat = "Roujin's challenge deactivated. Everything will be back to normal soon.",
+    norest_on_cheat = "Oh no! It appears your staff have consumed too much caffeine and no longer feel a need to rest.",
+    norest_off_cheat = "Phew! Looks like that buzz finally wore off. Your staff will now rest properly.",
+  },
+  staff_place_advice = {
+    not_enough_lecture_chairs = "Each student doctor needs a lecture chair to sit in!"
   },
 }
 
@@ -268,6 +273,7 @@ dynamic_info.staff.actions.heading_for = "Heading for %s"
 dynamic_info.staff.actions.fired = "Fired"
 dynamic_info.staff.actions.vaccine = "Vaccinating a patient"
 dynamic_info.patient.actions.epidemic_vaccinated = "I am no longer contagious"
+dynamic_info.object.strength_extra_info = "Strength %d (Upgradeable to %d)"
 
 progress_report.free_build = "FREE BUILD"
 
@@ -322,6 +328,7 @@ main_menu = {
   options = "Settings",
   map_edit = "Map Editor",
   savegame_version = "Savegame version: ",
+  updates_off = "Not checking for updates",
   version = "Version: ",
   exit = "Exit",
 }
@@ -393,12 +400,14 @@ tooltip.save_map_window = {
 menu_list_window = {
   name = "Name",
   save_date = "Modified",
+  ok = "OK",
   back = "Back",
 }
 
 tooltip.menu_list_window = {
   name = "Click here to sort the list by name",
   save_date = "Click here to sort the list by last modification date",
+  ok = "Confirm choice",
   back = "Close this window",
 }
 
@@ -406,6 +415,8 @@ options_window = {
   caption = "Settings",
   option_on = "On",
   option_off = "Off",
+  option_enabled = "Enabled",
+  option_disabled = "Disabled",
   fullscreen = "Fullscreen",
   resolution = "Resolution",
   capture_mouse = "Capture Mouse",
@@ -423,6 +434,7 @@ options_window = {
   shift_scrollspeed = "Shift Scroll Speed",
   zoom_speed = "Zoom Speed",
   hotkey = "Hotkeys",
+  check_for_updates = "Check for Updates",
 }
 
 tooltip.options_window = {
@@ -453,6 +465,7 @@ tooltip.options_window = {
   apply_zoomspeed = "Apply the entered zoom speed.",
   cancel_zoomspeed = "Return without changing the zoom speed.",
   hotkey = "Change keyboard hotkeys.",
+  check_for_updates = "Sets whether the game should look for updates on launch.",
 }
 
 customise_window = {
@@ -525,16 +538,16 @@ hotkey_window = {
   button_cancel = "Cancel",
   button_back = "Back",
   button_toggleKeys = "Toggle Keys",
+  button_gameSpeedKeys = "Game Speed Keys",
   button_recallPosKeys = "Recall Position Keys",
   panel_globalKeys = "Global Keys",
   panel_generalInGameKeys = "General In-Game Keys",
   panel_scrollKeys = "Scroll Keys",
   panel_zoomKeys = "Zoom Keys",
   panel_gameSpeedKeys = "Game Speed Keys",
-  panel_miscInGameKeys = "Misc. In-Game Keys",
   panel_toggleKeys = "Toggle Keys",
   panel_debugKeys = "Debug Keys",
-  panel_storePosKey = "Store Position Keys",
+  panel_storePosKeys = "Store Position Keys",
   panel_recallPosKeys = "Recall Position Keys",
   panel_altPanelKeys = "Alternate Panel Keys",
   global_confirm = "Confirm",
@@ -560,6 +573,7 @@ hotkey_window = {
   ingame_zoom_in_more = "Zoom In More",
   ingame_zoom_out = "Zoom Out",
   ingame_zoom_out_more = "Zoom Out More",
+  ingame_reset_zoom = "Reset Zoom",
   ingame_showmenubar = "Show Menu Bar",
   ingame_showCheatWindow = "Cheat Menu",
   ingame_loadMenu = "Load Game",
@@ -631,6 +645,7 @@ tooltip.hotkey_window = {
   button_defaults = "Reset all hotkeys to the program's defaults",
   button_cancel = "Cancel the assignment and go back to the options menu",
   caption_panels = "Open window to assign panel keys",
+  button_gameSpeedKeys = "Open window to set keys to control game speed",
   button_recallPosKeys = "Open window to set keys to store and recall camera positions",
   button_back_02 = "Go back to the main hotkey window. Hotkeys changed in this window can be accepted there",
 }
@@ -688,7 +703,6 @@ errors = {
   dialog_missing_graphics = "Sorry, the demo data files don't contain this dialog.",
   save_prefix = "Error while saving game: ",
   load_prefix = "Error while loading game: ",
-  compatibility_error = "Sorry, this save was created with a newer version of CorsixTH and is not compatible. Please update to a more recent version.",
   no_games_to_contine = "There are no saved games.",
   load_quick_save = "Error, cannot load the quicksave as it does not exist, not to worry as we have now created one for you!",
   map_file_missing = "Could not find the map file %s for this level!",
@@ -699,6 +713,12 @@ errors = {
   could_not_load_campaign = "Failed to load the campaign: %s",
   could_not_find_first_campaign_level = "Could not find the first level of this campaign: %s",
   save_to_tmp = "The file at %s could not be used. The game has been saved to %s. Error: %s",
+  dialog_empty_queue = "Sorry, a humanoid just had an empty action queue, which means that they didn't know what to do next. Please consult the command window for more detailed information. A dialog with the offending humanoid has been opened. Would you like them to leave the hospital?",
+  compatibility_error = {
+    new_in_old = "Sorry, this save was created with a newer version of CorsixTH and is not compatible. Please update to a more recent version.",
+    demo_in_full = "Sorry, you can't open a demo savegame with the full game files loaded. Please update your TH Data folder setting.",
+    full_in_demo = "Sorry, you can't open a full game save with the demo files loaded. Please update your TH Data folder setting.",
+  },
 }
 
 warnings = {
@@ -712,6 +732,7 @@ confirmation = {
   abort_edit_room = "You are currently building or editing a room. If all required objects are placed it will be finished, but otherwise it will be deleted. Continue?",
   maximum_screen_size = "The screen size you have entered is greater than 3000 x 2000. Larger resolutions are possible but will require better hardware in order to maintain a playable frame rate. Are you sure you want to continue?",
   remove_destroyed_room = "Would you like to remove the room for $%d?",
+  replace_machine_extra_info = "The new machine will have %d strength (currently %d).",
 }
 
 information = {
@@ -745,7 +766,7 @@ totd_window = {
     "Did you try to enter the European emergency number (112) into the fax machine? Make sure your sound is on!",
     "You can adjust some settings such as the resolution and language in the options window found both in the main menu and ingame.",
     "You selected a language other than English, but there's English text all over the place? Help us by translating missing texts into your language!",
-    "The CorsixTH team is looking for reinforcements! Are you interested in coding, translating or creating graphics for CorsixTH? Contact us at our Forum, Mailing List or IRC Channel (corsix-th at freenode).",
+    "The CorsixTH team is looking for reinforcements! Are you interested in coding, translating or creating graphics for CorsixTH? Contact us at our Discord Server, Sub-Reddit or Matrix Server. Links are on our website (CorsixTH.com).",
     "If you find a bug, please report it at our bugtracker: th-issues.corsix.org",
     "Each level has certain requirements to fulfill before you can move on to the next one. Check the status window to see your progression towards the level goals.",
     "If you want to edit or remove an existing room, you can do so with the edit room button found in the bottom toolbar.",
@@ -755,6 +776,7 @@ totd_window = {
     "Patients will get thirsty while waiting in your hospital, even more so if you turn up the heating! Place vending machines in strategic positions for some extra income.",
     "You can abort the diagnosis progress for a patient prematurely and guess the cure, if you already encountered the disease. Beware that this may increase the risk of a wrong cure, resulting in death for the patient.",
     "Emergencies can be a good source for some extra cash, provided that you have enough capacities to handle the emergency patients in time.",
+    "Did you know you can assign Handymen to specific plots? Just click the All Plots text in their staff profile to cycle through them!",
   },
   previous = "Previous Tip",
   next = "Next Tip",
@@ -769,6 +791,10 @@ debug_patient_window = {
   caption = "Debug Patient",
 }
 
+tooltip.debug_patient_window = {
+  item = "Create a debug patient with %s",
+}
+
 cheats_window = {
   caption = "Cheats",
   warning = "Warning: You will not get any bonus points at the end of the level if you cheat!",
@@ -888,28 +914,6 @@ map_editor_window = {
   }
 }
 
-fax = {
-vip_visit_result = {
-  ordered_remarks = {
-    [1] = "What a storming hospital. When I'm next seriously ill, take me there.",
-    [2] = "Now that's what I call a hospital.",
-    [3] = "That's a super hospital. And I should know; I've been in a few.",
-    [4] = "What a well-run hospital. Thanks for inviting me to it.",
-    [5] = "Hmm. Not a bad medical establishment, certainly.",
-    [6] = "I did enjoy your charming hospital. Now, anyone fancy a curry at the Taj?",
-    [7] = "Well, I've seen worse. But you should make some improvements.",
-    [8] = "Oh dear. Not a nice place to go if you're feeling peaky.",
-    [9] = "It's a standard hospital, to be honest. Frankly, I expected more.",
-    [10] = "Why did I bother? It was worse than going to see a four-hour opera!",
-    [11] = "I'm disgusted by what I saw. Call that a hospital? Pig-sty, more like!",
-    [12] = "I'm fed up of being in the public spotlight and visiting smelly holes like this! I resign.",
-    [13] = "What a dump. I'm going to try and get it closed down.",
-    [14] = "I have never seen such a dreadful hospital. What a disgrace!",
-    [15] = "I'm shocked. You can't call that a hospital! I'm off for a pint.",
-    }
-  }
-}
-
 hotkeys_file_err = {
   file_err_01 = "Unable to load hotkeys.txt file. Please ensure that CorsixTH " ..
         "has permission to read/write ",
@@ -923,3 +927,56 @@ transactions.remove_room = "Build: Remove destroyed room"
 
 options_window.change_resolution = "Change resolution"
 tooltip.options_window.change_resolution = "Change the window resolution to the dimensions entered on the left"
+
+--[[ Compatibility mapping for VIP result faxes in old saves (< 0.66). Using non-
+standard string formatting here, which should not be repeated in normal
+circumstances. This mapping will cause the legacy string to print in English but only
+for the relevant fax. These should be deleted on 2024 release. ]]--
+fax = {
+  vip_visit_result = {
+    ordered_remarks = {
+      [1] = fax.vip_visit_result.remarks[1],
+      [2] = fax.vip_visit_result.remarks[2],
+      [3] = fax.vip_visit_result.remarks[3],
+      [4] = fax.vip_visit_result.remarks[4],
+      [5] = fax.vip_visit_result.remarks[5],
+      [6] = fax.vip_visit_result.remarks[6],
+      [7] = fax.vip_visit_result.remarks[7],
+      [8] = fax.vip_visit_result.remarks[8],
+      [9] = fax.vip_visit_result.remarks[9],
+      [10] = fax.vip_visit_result.remarks[10],
+      [11] = fax.vip_visit_result.remarks[11],
+      [12] = fax.vip_visit_result.remarks[12],
+      [13] = fax.vip_visit_result.remarks[13],
+      [14] = fax.vip_visit_result.remarks[14],
+      [15] = fax.vip_visit_result.remarks[15],
+    },
+    remarks = {
+      super = {
+        fax.vip_visit_result.remarks[1],
+        fax.vip_visit_result.remarks[2],
+        fax.vip_visit_result.remarks[3],
+      },
+      good = {
+        fax.vip_visit_result.remarks[4],
+        fax.vip_visit_result.remarks[5],
+        fax.vip_visit_result.remarks[6],
+      },
+      mediocre = {
+        fax.vip_visit_result.remarks[7],
+        fax.vip_visit_result.remarks[8],
+        fax.vip_visit_result.remarks[9],
+      },
+      bad = {
+        fax.vip_visit_result.remarks[10],
+        fax.vip_visit_result.remarks[11],
+        fax.vip_visit_result.remarks[12],
+      },
+      very_bad = {
+        fax.vip_visit_result.remarks[13],
+        fax.vip_visit_result.remarks[14],
+        fax.vip_visit_result.remarks[15],
+      }
+    }
+  }
+}
