@@ -328,12 +328,12 @@ function Room:onHumanoidEnter(humanoid)
     self:tryAdvanceQueue()
     return
   end
-  local msg = {
+  local researcher_desks = {
     (_A.warnings.researcher_needs_desk_1),
     (_A.warnings.researcher_needs_desk_2),
     (_A.warnings.researcher_needs_desk_3),
   }
-  local msg_nurse = {
+  local nurse_desks = {
     (_A.warnings.nurse_needs_desk_1),
     (_A.warnings.nurse_needs_desk_2),
   }
@@ -345,10 +345,10 @@ function Room:onHumanoidEnter(humanoid)
         local staff_member = self:getStaffMember()
         self.humanoids[humanoid] = true
           if staff_member.profile.is_researcher and self.room_info.id == "research" then
-            self.world.ui.adviser:say(msg[math.random(1, #msg)])
+            self.hospital:giveAdvice(researcher_desks)
           end
           if staff_member.humanoid_class == "Nurse" and self.room_info.id == "ward" then
-            self.world.ui.adviser:say(msg_nurse[math.random(1, #msg_nurse)])
+            self.hospital:giveAdvice(nurse_desks)
           end
         if not staff_member.dealing_with_patient then
           staff_member:setNextAction(self:createLeaveAction())
@@ -493,18 +493,47 @@ function Room:commandEnteringStaff(humanoid, already_initialized)
   end
 end
 
+--! Activates and deactivates the staff waiting for patient mood icon
+-- and dynamic info text
+--!param activate (bool) - true to activate, false or nil to deactivate
+function Room:_staffWaitToggle(activate)
+  if not self.staff_member and not self.staff_member_set then
+    return -- No staff in room
+  end
+
+  local state = "deactivate"
+  local dynamic_text = ""
+
+  if activate then
+    dynamic_text = _S.dynamic_info.staff.actions.waiting_for_patient
+    state = "activate"
+  end
+
+  if not self.staff_member_set and self.staff_member then
+    self.staff_member:setMood("staff_wait", state)
+    self.staff_member:setDynamicInfoText(dynamic_text)
+  else
+    for staff_member in pairs(self.staff_member_set) do
+      staff_member:setMood("staff_wait", state)
+      staff_member:setDynamicInfoText(dynamic_text)
+    end
+  end
+end
+
+--! Check the target room for a staff wait toggle is not staff room/training/toilets,
+--! and the front humanoid is a patient.
+function Room:_checkWaitToggleValidTarget()
+  return self:hasQueueDialog() and self.room_info.id ~= "toilets" and
+      class.is(self.door.queue:front(), Patient)
+end
+
 function Room:commandEnteringPatient(humanoid)
   -- To be extended in derived classes
   self.door.queue.visitor_count = self.door.queue.visitor_count + 1
   humanoid:updateDynamicInfo("")
 
-  for room_humanoid in pairs(self.humanoids) do -- Staff is no longer waiting
-    if class.is(room_humanoid, Staff) then
-      if room_humanoid.humanoid_class ~= "Handyman" then
-        room_humanoid:setMood("staff_wait", "deactivate")
-        room_humanoid:setDynamicInfoText("")
-      end
-    end
+  if self:_checkWaitToggleValidTarget() then
+    self:_staffWaitToggle(false) -- Staff no longer waiting
   end
 end
 
@@ -516,16 +545,8 @@ function Room:tryAdvanceQueue()
     if self:canHumanoidEnter(front) then
       self.door.queue:pop()
       self.door:updateDynamicInfo()
-      -- Do nothing if it is the staff room or training room.
-      if self:hasQueueDialog() then
-        for room_humanoid in pairs(self.humanoids) do -- Staff is now waiting
-          if class.is(room_humanoid, Staff) then
-            if room_humanoid.humanoid_class ~= "Handyman" then
-              room_humanoid:setMood("staff_wait", "activate")
-              room_humanoid:setDynamicInfoText(_S.dynamic_info.staff.actions.waiting_for_patient)
-            end
-          end
-        end
+      if self:_checkWaitToggleValidTarget() then
+        self:_staffWaitToggle(true) -- Staff are now waiting
       end
     elseif self.humanoids[front] then
       self.door.queue:pop()
@@ -534,6 +555,8 @@ function Room:tryAdvanceQueue()
   end
 end
 
+--! Handles the departure of a humanoid from the room
+--!param humanoid The subject entity
 function Room:onHumanoidLeave(humanoid)
   if self.staff_member == humanoid then
     self.staff_member = nil
@@ -544,7 +567,7 @@ function Room:onHumanoidLeave(humanoid)
     return
   end
   self.humanoids[humanoid] = nil
-  local staff_leaving = false
+  local staff_leaving = false -- used only for staff leaving immediately after a patient
 
   if class.is(humanoid, Patient) then
     -- Some staff member in the room might be waiting to get to the staffroom.
@@ -583,18 +606,20 @@ function Room:onHumanoidLeave(humanoid)
       humanoid.waiting_on_other_staff = nil
       self.dealt_patient_callback = nil
     end
-    -- Make patients leave the room if there are no longer enough staff
+    -- Make patients leave the room (except wards) if there are no longer enough staff
     if not self:testStaffCriteria(self:getRequiredStaffCriteria()) then
-      local patient_needs_to_reenter = false
+      local call_for_new_staff = self.door.queue:patientSize() > 0
       for room_humanoid in pairs(self.humanoids) do
         if class.is(room_humanoid, Patient) and self:shouldHavePatientReenter(room_humanoid) then
-          self:makeHumanoidLeave(room_humanoid)
-          room_humanoid:queueAction(self:createEnterAction(room_humanoid))
-          patient_needs_to_reenter = true
+          call_for_new_staff = true
+          if self.room_info.id ~= "ward" then
+            self:makeHumanoidLeave(room_humanoid)
+            room_humanoid:queueAction(self:createEnterAction(room_humanoid))
+          end
         end
       end
       -- Call for staff if needed
-      if self.is_active and (patient_needs_to_reenter or self.door.queue:patientSize() > 0) then
+      if self.is_active and call_for_new_staff then
         self.world.dispatcher:callForStaff(self)
       end
     end
@@ -935,7 +960,7 @@ function Room:deactivate()
 
   -- crashRoom might have deactivated the door already
   if self.door.queue then
-    self.door.queue:rerouteAllPatients(SeekRoomAction(self.room_info.id))
+    self.door.queue:rerouteAllPatients(self.room_info.id)
   end
 
   self.hospital:removeRatholesAroundRoom(self)
