@@ -149,8 +149,7 @@ function UI:UI(app, minimal)
   if minimal then
     self.tooltip_font = app.gfx:loadBuiltinFont()
   else
-    local palette = app.gfx:loadPalette("QData", "PREF01V.PAL")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
+    local palette = app.gfx:loadPalette("QData", "PREF01V.PAL", true)
     self.tooltip_font = app.gfx:loadFont("QData", "Font00V", false, palette)
   end
   self.tooltip = nil
@@ -195,57 +194,16 @@ function UI:UI(app, minimal)
 
   self:setCursor(self.default_cursor)
 
-  -- to avoid a bug which causes open fullscreen windows to display incorrectly, load
-  -- the sprite sheet associated with all fullscreen windows so they are correctly cached.
-  -- Darrell: Only do this if we have a valid data directory otherwise we won't be able to
-  -- display the directory browser to even find the data directory.
-  -- Edvin: Also, the demo does not contain any of the dialogs.
-  if self.app.good_install_folder and not self.app.using_demo_files then
-    local gfx = self.app.gfx
-    local palette
-    -- load drug casebook sprite table
-    palette = gfx:loadPalette("QData", "DrugN01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "DrugN02V", true, palette)
-    -- load fax sprite table
-    palette = gfx:loadPalette("QData", "Fax01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Fax02V", true, palette)
-    -- load town map sprite table
-    palette = gfx:loadPalette("QData", "Town01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Town02V", true, palette)
-    -- load hospital policy sprite table
-    palette = gfx:loadPalette("QData", "Pol01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Pol02V", true, palette)
-    -- load bank manager sprite table
-    palette = gfx:loadPalette("QData", "Bank01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Bank02V", true, palette)
-    -- load research screen sprite table
-    palette = gfx:loadPalette("QData", "Res01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Res02V", true, palette)
-    -- load progress report sprite table
-    palette = gfx:loadPalette("QData", "Rep01V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Rep02V", true, palette)
-    -- load annual report sprite table
-    palette = gfx:loadPalette("QData", "Award02V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
-    gfx:loadSpriteTable("QData", "Award03V", true, palette)
-  end
 
   self:setupGlobalKeyHandlers()
 end
 
 function UI:runDebugScript()
+  -- luacheck: ignore 111 _ is set in debug code
   print("Executing Debug Script...")
-  local path_sep = package.config:sub(1, 1)
-  local lua_dir = debug.getinfo(1, "S").source:sub(2, -8)
+  local debug_script = self.app:getFullPath({"Lua", "debug_script.lua"})
   _ = TheApp.ui and TheApp.ui.debug_cursor_entity
-  local script = assert(loadfile(lua_dir .. path_sep .. "debug_script.lua"))
+  local script = assert(loadfile(debug_script))
   script()
   -- Clear _ after the script to prevent save corruption
   _ = nil
@@ -258,7 +216,7 @@ function UI:setupGlobalKeyHandlers()
   self:addKeyHandler("global_stop_movie", self, self.stopMovie)
   self:addKeyHandler("global_stop_movie_alt", self, self.stopMovie)
   self:addKeyHandler("global_screenshot", self, self.makeScreenshot)
-  self:addKeyHandler("global_fullscreen_toggle", self, self.toggleFullscreen)
+  self:addKeyHandler("global_fullscreen_toggle", self, self.fullscreenHotkey)
   self:addKeyHandler("global_exitApp", self, self.exitApplication)
   self:addKeyHandler("global_resetApp", self, self.resetApp)
   self:addKeyHandler("global_releaseMouse", self, self.releaseMouse)
@@ -280,11 +238,8 @@ function UI:playSound(name, played_callback, played_callback_delay)
   end
 end
 
--- Used for announcements only
+-- Stub with args for subclass GameUI.
 function UI:playAnnouncement(name, priority, played_callback, played_callback_delay)
-  if self.app.config.play_announcements then
-    self.app.audio:playSound(name, nil, true, played_callback, played_callback_delay)
-  end
 end
 
 function UI:setDefaultCursor(cursor)
@@ -364,7 +319,7 @@ end
 --!param ... Additional arguments to `callback`.
 function UI:addKeyHandler(keys, window, callback, ...)
   -- It is necessary to clone the key table into another temporary table, as if we don't the original table that we take it from will lose
-  -- the last key of that table permenently in the next line of code after this one, until the program is restarted.
+  -- the last key of that table permanently in the next line of code after this one, until the program is restarted.
   -- I.E. if the "ingame_quitLevel" hotkey from the "hotkeys_values" table in "config_finder.lua" is a table that looks like this:
   --   {"shift", "q"}
   -- We would lose the "q" element until we restarted the game and the "hotkey.txt" was read from again, causing the "ingame_quitLevel"
@@ -593,6 +548,7 @@ function UI:onChangeResolution()
   for _, window in ipairs(self.windows) do
     window:onChangeResolution()
   end
+  self.app.audio:setSoundStage()
 end
 
 function UI:registerTextBox(box)
@@ -673,6 +629,23 @@ function UI:releaseMouse()
   self:setMouseReleased(true)
 end
 
+--! Dedicated hotkey function for toggling fullscreen
+function UI:fullscreenHotkey()
+  local toggle = self:toggleFullscreen()
+  if not toggle then
+    local err = {_S.errors.unavailable_screen_size}
+    self:addWindow(UIInformation(self, err))
+  end
+  -- Update the Options window, if open
+  local window = self:getWindow(UIOptions)
+  if window then
+    if toggle then window.fullscreen_button:toggle() end
+    window.fullscreen_panel:setLabel(self.app.fullscreen and _S.options_window.option_on or _S.options_window.option_off)
+  end
+end
+
+--! Turns fullscreen on and off
+--!return success true if toggle succeeded
 function UI:toggleFullscreen()
   local modes = self.app.modes
 
@@ -727,8 +700,7 @@ end
 
 --! Called when the user presses a key on the keyboard
 --!param rawchar (string) The name of the key the user pressed.
---!param is_repeat (boolean) True if this is a key repeat event
-function UI:onKeyDown(rawchar, modifiers, is_repeat)
+function UI:onKeyDown(rawchar, modifiers)
   local handled = false
   -- Apply key-remapping and normalisation
   rawchar = string.sub(rawchar,1,6) == "Keypad" and
@@ -1101,6 +1073,17 @@ function UI:afterLoad(old, new)
   if old < 5 then
     self.editing_allowed = true
   end
+  if old < 179 then
+    if self.app.good_install_folder and not self.app.using_demo_files then
+      local gfx = self.app.gfx
+      gfx.cache.raw = {}
+      gfx.cache.tabled = {}
+      gfx.cache.palette = {}
+      gfx.cache.palette_greyscale_ghost = {}
+      gfx.cache.language_fonts = {}
+      gfx.builtin_font = nil
+    end
+  end
   self:setupGlobalKeyHandlers()
 
   -- Cancel any saved screen movement from edge scrolling
